#!/usr/bin/perl
use lib '/home/martin/ethereal/mgmt';

################################################################################
# Created       : Martin Foster
# Modified      : 16-Mar-2006
################################################################################
#
# Config Realm - Script part of Ethereal Realms, designed for supervisors
#                to control realms
# Copyright (C) 2000-2006  Martin Foster
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# Author of this script can be contacted at the following:
#       E-Mail  : martin@ethereal-realms.org
#       Address : 29-26503 TWP RD 511
#                 Spruce Grove, Alberta
#                 T7Y 1G4
#
#################################################################################

use DB_File;							# BerkelyDB handle
use CGI;							# Common gateway interface
use CGI::Carp qw(fatalsToBrowser);				# CGI Error logs
use LWP::Simple qw(!head);					# Simple web space access
use XML::RSS;							# RSS handling capabilites
use Storable qw(nfreeze thaw);					# Storable features
use strict;							# Strict variable enforcement

use Ethereal::Audit;						# Auditing information
use Ethereal::Comm;						# Comm panel
use Ethereal::Database;						# Database handler
use Ethereal::Filter;						# Filtering handle
use Ethereal::Login;						# Login functionality
use Ethereal::Mail;						# Mail handler
use Ethereal::Option;						# Option handler
use Ethereal::Param;						# Parameter control
use Ethereal::Pull;						# Post retrieval
use Ethereal::Template;						# Template handler
use Ethereal::Weather;						# Weather handler
use Ethereal::Who;						# Who is on

#################################################################################
# Gobal override
#################################################################################
$CGI::POST_MAX=1024 * 50;					# Maximum posts
$CGI::DISABLE_UPLOADS = 1;					# Disable uploads

################################################################################
# Data Members
################################################################################
my $cgi;							# Common gateway interface handle
my $database;							# Database handle
my $login;							# Login handle
my $param;							# Parameter handle
my $pull;							# Post retreival
my $tmpl;							# Template handle

my $admin;							# Administrative override
my $access;							# Realm access
my $acl;							# Access control list
my $host = 15;							# Requirement to be over for host
my $links;							# Link management
my $url;							# Self referencing link
my $verdict;							# Verdict on bans
my $size = 0;							# Size pulled

my @acl;							# Captured ACL

my %args;							# Default arguments
my %config;							# Configuration hash


################################################################################
# Program Area
################################################################################

	# Initial handle
	$cgi      = new CGI;	
	$database = new Ethereal::Database();
	$pull     = new Ethereal::Pull();

	# Disabled caching for safety
	$database->{'ADMN'} = 'true';

	# Connect and fetch
	$database->Connect($cgi);


	# Pre-capture
	@acl = $cgi->param('MODACL');


	# Parameter handling
	$param = new Ethereal::Param($database, $cgi);
	$param->GetParam();

	# Fetch user
	$url = $cgi->url(-absolute=>1);

	# Fetch hash
	$database->GetHashConfiguration(\%config);

	# Create instances
	$tmpl     = new Ethereal::Template(\%config);


	# Login handle
	$login = new Ethereal::Login($database, $cgi, $param);

	# Enable custom header and footer
	$login->{'HEADER'} = $database->DocumentGetSuperHeader();
	$login->{'FOOTER'} = $database->DocumentGetSuperFooter();
	

	# Authenthication
	if ($login->GetVerificationSuper())
	{
		# Document header
		print $login->{'HEADER'}, "\n";

		# Fetch Posts
		$pull->AdminPosts($database, $cgi, $param);

		# Realm selection
		unless (defined($param->{'ROOM'}))
		{
			# Have to select a realm
			RealmSelect($database, $cgi, $param, $tmpl, \%config)
		}

		# Realm handling
		else
		{ 
			# Prevent bans from use
			($access)  = $database->DataGetRealmAccess($param->{'ROOM'});
			($acl)     = $database->DataGetRealmACL($param->{'USER'}, $param->{'ROOM'});
			($verdict) = $database->DataGetBannedStatus($param->{'USER'}, $param->{'ROOM'});

			# Administrative access
			($admin)   = $database->DataGetAdmin($param->{'USER'}); 


			# Only show if not banned
			unless (((($access eq 'relaxed') && (defined($verdict)))
			      or (($access eq 'restricted') && (defined($verdict)))
			      or (($access eq 'list') && (!defined($verdict))))
			     and ($admin eq 'no'))
			{
				# Generate arguments
				%args = (
					USER  => $param->{'USER'},
					CRYPT => $param->{'CRYPT'}
				 );


				# Generate inlines
				# Back button
				my ($iback) = $param->EmbedInline(%args); 


				# Add parameter
				$args{'ROOM'} = $param->{'ROOM'};

				# Regular inlines
				my ($ibckp) = $param->EmbedInline(%args, BACKUP   => 'True');
				my ($ibann) = $param->EmbedInline(%args, BANN     => 'True');
				my ($icnon) = $param->EmbedInline(%args, CANON    => 'True');
				my ($idsgn) = $param->EmbedInline(%args, DESIGN   => 'True');
				my ($ifeed) = $param->EmbedInline(%args, FEED     => 'True');
				my ($ifntc) = $param->EmbedInline(%args, FONT     => 'True');
				my ($iforc) = $param->EmbedInline(%args, FORCAST  => 'True');
				my ($ipics) = $param->EmbedInline(%args, GRAPH    => 'True');
				my ($igumm) = $param->EmbedInline(%args, GUMMY    => 'True');
				my ($ilocl) = $param->EmbedInline(%args, LOCALE   => 'True');
				my ($imods) = $param->EmbedInline(%args, MODS     => 'True');
				my ($itopc) = $param->EmbedInline(%args, TOPIC    => 'True');
				my ($iwarn) = $param->EmbedInline(%args, WARN     => 'True');
				my ($iword) = $param->EmbedInline(%args, WORD     => 'True');
				my ($ipost) = $param->EmbedInline(%args, GETPOSTS => 'True');
				my ($irelm) = $param->EmbedInline(%args);


				# Initialize
				$links = length($config{'TagBack'});
				

				# Display Links
				# Normal supervisory display
				if ($acl > $host)
				{
					# Display every option
					$links = $tmpl->Pass('TagTabbed',
							MLINK => $url.$iback,
							MNAME => $config{'TagBack'})

					       . Spacer(\$size, $config{'SetLimit'}, $config{'TagRealm'})
						
					       . $tmpl->Pass('TagTabbed',
						        MLINK => $url.$irelm,
							MNAME => $config{'TagRealm'})

					       . Spacer(\$size, $config{'SetLimit'}, $config{'TagBackup'})
						
					       . $tmpl->Pass('TagTabbed', 
						       MLINK => $url.$ibckp,
						       MNAME => $config{'TagBackup'})

					       . Spacer(\$size, $config{'SetLimit'}, $config{'TagBann'})
						
					       . $tmpl->Pass('TagTabbed', 
						       MLINK => $url.$ibann,
						       MNAME => $config{'TagBann'})

					       . Spacer(\$size, $config{'SetLimit'}, $config{'TagTopic'})
					       
					       . $tmpl->Pass('TagTabbed',
						       MLINK => $url.$itopc,
						       MNAME => $config{'TagTopic'})

					       . Spacer(\$size, $config{'SetLimit'}, $config{'TagCanon'})
					       
					       . $tmpl->Pass('TagTabbed',
						       MLINK => $url.$icnon,
						       MNAME => $config{'TagCanon'})

					       . Spacer(\$size, $config{'SetLimit'}, $config{'TagDesign'})
					       
					       . $tmpl->Pass('TagTabbed',
						       MLINK => $url.$idsgn,
						       MNAME => $config{'TagDesign'})

					       . Spacer(\$size, $config{'SetLimit'}, $config{'TagFeeds'})
					       
					       . $tmpl->Pass('TagTabbed',
						       MLINK => $url.$ifeed,
						       MNAME => $config{'TagFeeds'})

					       . Spacer(\$size, $config{'SetLimit'}, $config{'TagFont'})
					       
					       . $tmpl->Pass('TagTabbed',
						       MLINK => $url.$ifntc,
						       MNAME => $config{'TagFont'})

					       . Spacer(\$size, $config{'SetLimit'}, $config{'TagForcast'})
					       
					       . $tmpl->Pass('TagTabbed',
						       MLINK => $url.$iforc,
						       MNAME => $config{'TagForcast'})

					       . Spacer(\$size, $config{'SetLimit'}, $config{'TagGummy'})
					       
					       . $tmpl->Pass('TagTabbed',
						       MLINK => $url.$igumm,
						       MNAME => $config{'TagGummy'})

					       . Spacer(\$size, $config{'SetLimit'}, $config{'TagLocale'})
					       
					       . $tmpl->Pass('TagTabbed',
						       MLINK => $url.$ilocl,
						       MNAME => $config{'TagLocale'})

					       . Spacer(\$size, $config{'SetLimit'}, $config{'TagMods'})
					       
					       . $tmpl->Pass('TagTabbed',
						       MLINK => $url.$imods,
						       MNAME => $config{'TagMods'})

					       . Spacer(\$size, $config{'SetLimit'}, $config{'TagPics'})
					       
					       . $tmpl->Pass('TagTabbed',
						       MLINK => $url.$ipics,
						       MNAME => $config{'TagPics'})

					       . Spacer(\$size, $config{'SetLimit'}, $config{'TagPosts'})
					       
					       . $tmpl->Pass('TagTabbed',
						       MLINK => $url.$ipost,
						       MNAME => $config{'TagPosts'})

					       . Spacer(\$size, $config{'SetLimit'}, $config{'TagWarn'})
					       
					       . $tmpl->Pass('TagTabbed',
						       MLINK => $url.$iwarn,
						       MNAME => $config{'TagWarn'})

					       . Spacer(\$size, $config{'SetLimit'}, $config{'TagWord'})
					       
					       . $tmpl->Pass('TagTabbed',
						       MLINK => $url.$iword,
						       MNAME => $config{'TagWord'});
				}


				# Moderative 'Me Too!'
				# These will adjust accordingly
				else
				{
					# General option to back out
					$links = $tmpl->Pass('TagTabbed',
						        MLINK => $url.$iback,
							MNAME => $config{'TagBack'});


					# Storyteller
					if (($acl - 8) >= 0)
					{
						# Remove amount
						$acl -= 8;

						# Options
						$links .= Spacer(\$size, $config{'SetLimit'}, $config{'TagCanon'})

						       . $tmpl->Pass('TagTabbed',
								 MLINK => $url.$icnon,
								 MNAME => $config{'TagCanon'})

						       . Spacer(\$size, $config{'SetLimit'}, $config{'TagGummy'})

						       . $tmpl->Pass('TagTabbed',
							         MLINK => $url.$igumm,
								 MNAME => $config{'TagGummy'})

						       . Spacer(\$size, $config{'SetLimit'}, $config{'TagLocale'})

						       . $tmpl->Pass('TagTabbed',
							         MLINK => $url.$ilocl,
								 MNAME => $config{'TagLocale'});
					}

					# Jailer
					if (($acl - 4) >= 0)
					{
						# Remove amount
						$acl -= 4;

						# Options
						$links .= Spacer(\$size, $config{'SetLimit'}, $config{'TagBann'})

						       . $tmpl->Pass('TagTabbed',
								 MLINK => $url.$ibann,
								 MNAME => $config{'TagBann'})

						       . Spacer(\$size, $config{'SetLimit'}, $config{'TagWord'})
							 
						       . $tmpl->Pass('TagTabbed',
							         MLINK => $url.$iword,
								 MNAME => $config{'TagWord'});
					}

					# Designer
					if (($acl - 2) >= 0)
					{
						# Remove amount
						$acl -= 2;

						# Options
						$links .= Spacer(\$size, $config{'SetLimit'}, $config{'TagDesign'})

						       . $tmpl->Pass('TagTabbed',
								 MLINK => $url.$idsgn,
								 MNAME => $config{'TagDesign'})

						       . Spacer(\$size, $config{'SetLimit'}, $config{'TagFont'})

						       . $tmpl->Pass('TagTabbed',
								 MLINK => $url.$ifntc,
								 MNAME => $config{'TagFont'});
					}

					# Archivist
					if (($acl - 1) >= 0)
					{
						# One and only option
						$links .= Spacer(\$size, $config{'SetLimit'}, $config{'TagPosts'})

						       . $tmpl->Pass('TagTabbed',
								 MLINK => $url.$ipost,
								 MNAME => $config{'TagPosts'});
					}


					# Everyone gets to play
					$links .= Spacer(\$size, $config{'SetLimit'}, $config{'TagWarn'})

					       . $tmpl->Pass('TagTabbed',
							 MLINK => $url.$iwarn,
							 MNAME => $config{'TagWarn'});
				}


				# Realm information
				RoomWrite($database, $cgi, $param, \%config, $access, $admin, \@acl);


				# Interface management
				# Backup and restore
				if ((defined($param->{'BACKUP'}))
				 && ($acl > $host))
				{
					Backup($database, $cgi, $param, $tmpl, \%config, $links);
				}

				# Bannings or access pool
				elsif (defined($param->{'BANN'}))
				{
					RoomBan($database, $cgi, $param, $tmpl, \%config, $links, $access);
				}


				# Canon handles
				elsif (defined($param->{'CANON'}))
				{
					RoomCanon($database, $cgi, $param, $tmpl, \%config, $links);
				}

				# Realm Design
				elsif (defined($param->{'DESIGN'}))
				{
					RoomDesign($database, $cgi, $param, $tmpl, \%config, $url, $links);
				}

				# Feed handler
				elsif (defined($param->{'FEED'}))
				{
					FeedHandler($database, $cgi, $param, $tmpl, \%config, $links);
				}
				
				# Font handler
				elsif (defined($param->{'FONT'}))
				{
					FontHandler($database, $cgi, $param, $tmpl, \%config, $links);
				}

				# Gummy handler
				elsif (defined($param->{'GUMMY'}))
				{
					GummyHandler($database, $cgi, $param, $tmpl, \%config, $links);	
				}

				# Locale handler
				elsif (defined($param->{'LOCALE'}))
				{
					LocaleHandler($database, $cgi, $param, $tmpl, \%config, $links);	
				}

				# Weather Forcasts
				elsif ((defined($param->{'FORCAST'}))
				 && ($acl > $host))
				{
					Forcasts($database, $cgi, $param, $tmpl, \%config, $links);
				}

				# Picture Restriction
				elsif ((defined($param->{'GRAPH'}))
				 && ($acl > $host))
				{
					RoomPicture($database, $cgi, $param, $tmpl, \%config, $links);
				}

				# Moderator Handling
				elsif ((defined($param->{'MODS'}))
				 && ($acl > $host))
				{
					RoomMods($database, $cgi, $param, $tmpl, \%config, $links);
				}
			

				# Board Topics
				elsif (defined($param->{'TOPIC'}))
				{
					BoardTopic($database, $cgi, $param, $tmpl, \%config, $links);
				}

				# Word Restrictions
				elsif (defined($param->{'WARN'}))
				{
					RoomWarn($database, $cgi, $param, $tmpl, \%config, $links);
				}

				# Word Restrictions
				elsif (defined($param->{'WORD'}))
				{
					RoomWord($database, $cgi, $param, $tmpl, \%config, $links);
				}


				# Realm information
				else
				{
					# Default for supervisors
					if ($acl > $host)
					{
						# Allow them to see what is necessary
						RoomView($database, $cgi, $param, $tmpl, \%config, $admin, $links);	
					}
					else
					{
						# Display message
						$tmpl->Show('TmplStart',
							MLINKS => $links
						 );
					}
				}
			}
		}

		# Document footer
		print $login->{'FOOTER'}, "\n";
	}


################################################################################
# Sub-Routines
################################################################################

#####################
# Backup
#
# Backup and restore capabilities for Ethereal Realms in order to secure
# designs.   

sub Backup
{
	#####################
	# Data members
	my $database = shift;					# Database handle
	my $cgi      = shift;					# CGI handle
	my $param    = shift;					# Parameter handle
	my $tmpl     = shift;					# Template handle
	my $config   = shift;					# Config hash
	my $links    = shift;					# Links to display

	my $statement;						# Query statement
	my $res;						# Results handle

	my $count    = 0;					# Counter
	my $key;						# Restore key
	my $wrap     = '72';					# Wrapping length

	my $import   = '';					# Import values	
	my $export   = '';					# Exported values
	my $wrapped  = '';					# Wrapped content

	my %backup;						# Backup hash
	my %restore;						# Restore hash


	#####################
	# Program Area

	# Update information
	if (defined($param->{'CONFIRM'})
	 && (defined($param->{'EXPORT'}))
	 && (length($param->{'EXPORT'}) > 25))
	{
		# Massage
		$import =  $param->{'EXPORT'};
		$import =~ s/(\s|\n|$)//gs;


		# Attempt to recover
		eval
		{
			# Recover line
			%restore = %{thaw(pack('H*', $import))};
		};

		# No errors continue
		unless ($@)
		{
			# Remove all entries
			$database->Quick("DELETE FROM RealmDesign
				WHERE RealmName=?", $param->{'ROOM'}
			 );


			# Cycle through
			# Recreate all templates as they were
			foreach $key (keys(%restore))
			{
				# Create all new entries
				$database->Quick("INSERT INTO RealmDesign
					(RealmName,
					 TagName,
					 DesignContent)
					VALUES(?,?,?)",

				 $param->{'ROOM'},
				 $key,
				 $restore{$key}
				);	
			}
		}
	}
	
	# Cleanup
	$param->Cleanup('EXPORT', 'CONFIRM');


	# General information gathering
	# Prepare query
	$database->Pull(\$statement, "SELECT
		  TagName       AS \"TagName\",
		  DesignContent AS \"DesignContent\"
		FROM  RealmDesign
		WHERE RealmName=?
		ORDER BY TagName", $param->{'ROOM'}
	 );

	# Loop and execute
	if ($res = $statement->fetchrow_hashref())
	{
		# Loop until task is complete
		do
		{
			# Populate properly
			$backup{$res->{'TagName'}} = $res->{'DesignContent'};
		}
		while ($res = $statement->fetchrow_hashref());

		# Export information
		$export = unpack('H*', nfreeze(\%backup));


		# Rewrap
		# Cut through
		for ($count=0; $count <= (int(length($export) / $wrap)); $count++)
		{
			# Cut portion of text
			$wrapped .= substr($export, ($count*$wrap), $wrap) . "\n";
		}
	}

	# Complete query
	$statement->finish();


	# HTML Display
	# Form starts
	print $cgi->start_form();
	print $param->EmbedNormal($param->Flat(), BACKUP=>'True'), "\n";

	# Display template	
	$tmpl->Show('TmplBackup',
		MLINKS   => $links,
		WRESTORE => $cgi->submit($config->{'TxtSubmitRestore'}),
		WCONFIRM => $cgi->checkbox(
				-name    => 'CONFIRM',
				-label   => ''),
		WEXPORT  => $cgi->textarea(
				-name    => 'EXPORT',
				-default => $wrapped,
				-rows    => 10,
				-columns => 75,
				-wrap    => 'SOFT')
	  );

	# End of form
	print $cgi->end_form(), "\n";
}

#####################
# Message Board Topic
#
# Allows for the addition and removal of topics in relation to their use in
# the message boards integrated with the system itself.

sub BoardTopic
{
	#####################
	# Data members
	my $database = shift;					# Database handle
	my $cgi      = shift;					# CGI handle
	my $param    = shift;					# Parameter handle
	my $tmpl     = shift;					# Template handle
	my $config   = shift;					# Config hash
	my $links    = shift;					# Links to display

	my $descr;						# Descriptions
	my $select;						# Select widgit
	my $topics;						# List of topics

	my @topics;						# List of topics	


	#####################
	# program area

	# Shared aspects
	# Retreive list of banned and suspects
	$database->GetListTopics(\@topics, $param->{'ROOM'});

	# Append additional
	unshift(@topics, $config->{'TxtTopic'});
		

	# Create widgets
	# Topics
	$topics = $cgi->popup_menu('TOPIC',  \@topics, $param->{'TOPIC'});

	# Options control
	$select = $cgi->submit($config->{'TxtSubmitView'}); 


	# Differences
	# Creation screen
	if (($param->{'TOPIC'} eq 'True')
	 || ($param->{'TOPIC'} eq $config->{'TxtTopic'}))
 	{
		# HTML
		# Beginning of form
		print $cgi->start_form();
		print $param->EmbedNormal($param->Flat(), TOPICS=>'True'), "\n";


		# Display template
		$tmpl->Show('TmplBoardCreate',
			MLINKS  => $links,
			WTOPICS => $topics,
			WSELECT => $select,
			WNAME   => $cgi->textfield('TADD', '', 15, 25),
			WCREATE => $cgi->submit($config->{'TxtSubmitAddition'}),
			WDESCR  => $cgi->textarea(
					-name    => 'TDESCR',
					-default => '',
					-rows    => 3,
					-columns => 50,
					-wrap    => 'SOFT')
		 );
			
		# End of form
		print $cgi->end_form(), "\n";
	}
		

	# Base management screen
	else
	{
		# Pull details
		($descr) = $database->DataGet("SELECT BoardDescription
			FROM BoardTopic
			WHERE BoardTopic=?
			  AND RealmName=?",

		 $param->{'TOPIC'},
		 $param->{'ROOM'}
		);	

		# Display
		# Beginning of form
		print $cgi->start_form();
		print $param->EmbedNormal($param->Flat(), TOPICS=>'True', TOPIC=>$param->{'TOPIC'}), "\n";

		# Display template
		$tmpl->Show('TmplBoardManage',
			MLINKS  => $links,
			MTOPIC  => $param->{'TOPIC'},
			WTOPICS => $topics,
			WSELECT => $select,
			WUPDATE => $cgi->submit($config->{'TxtSubmitAlter'}),
			WDELETE => $cgi->checkbox(
					-name    => 'TDELETE',
					-label   => ''),
			WDESCR  => $cgi->textarea(
					-name    => 'TEDIT',
					-default => $descr,
					-rows    => 3,
					-columns => 50,
					-wrap    => 'SOFT')
		 );

		# End of form
		print $cgi->end_form(), "\n";
	}
}


#####################
# Feed Handler
#
# Will display both the detailed and normal view of feeds based on the
# information that is has received.

sub FeedHandler
{
	#####################
	# Data members
	my $database = shift;					# Database handle
	my $cgi      = shift;					# CGI handle
	my $param    = shift;					# Parameter hash
	my $tmpl     = shift;					# Template handle
	my $config   = shift;					# Config hash
	my $links    = shift;					# Link management

	my $res;						# Results set
	my $statement;						# Database statement
	
	my $code;						# Code to display
	my $id;							# Feed identifier

	my $buttons;						# Buttons widget
	my $feeds;						# RSS feed widget

	my @feeds;						# List of RSS feeds
	my @options  = qw(Yes No);				# Choice of the matter


	#####################
	# Program area

	# Detailed listing
	if (($param->{'FEED'} ne 'True')
	 && ($param->{'FEED'} ne $config->{'TxtNone'}))
	{
		# Pull ID and Topic
		$param->{'FEED'} =~ /(\d+) - (.+)/;

		# Assign
		my $id   = $1;

		# Retrieve information
		$database->Pull(\$statement, "SELECT
		 	 FeedID           AS \"FeedID\",
			 FeedTitle        AS \"FeedTitle\",
			 FeedLink         AS \"FeedLink\",
			 FeedSource       AS \"FeedSource\",
			 FeedMax          AS \"FeedMax\",
			 FeedCompactView  AS \"FeedCompactView\",
			 FeedShowDate     AS \"FeedShowDate\",
			 FeedNewWindow    AS \"FeedNewWindow\",
			 getDateTime(FeedTimestamp) AS \"FeedTimestamp\"
			FROM Feed
			WHERE FeedID=?
			  AND RealmName=?",

		 $id,
		 $param->{'ROOM'}
		);

		# Validity test
		if ($res = $statement->fetchrow_hashref())
		{
			# Generated code
			$code =  $config->{'SetFeedCode'};
			$code =~ s/MID/$res->{'FeedID'}/gs;

			
			# Generate widgets
			my $view = $cgi->radio_group('FEEDVIEW', \@options, ucfirst($res->{'FeedCompactView'}), 'true');
			my $date = $cgi->radio_group('FEEDDATE', \@options, ucfirst($res->{'FeedShowDate'}), 'true');
			my $neww = $cgi->radio_group('FEEDNEWW', \@options, ucfirst($res->{'FeedNewWindow'}), 'true');
			
			# HTML
			# Beginning of form
			print $cgi->start_form();
			print $param->EmbedNormal($param->Flat(), FID=>$res->{'FeedID'}), "\n";

			# Display template
			$tmpl->Show('TmplFeedDetails',
				MFEED     => $res->{'FeedTitle'},
				MLINK     => $res->{'FeedSource'},
				MARCHIVE  => $res->{'FeedTimestamp'},
				WTITLE    => $cgi->textfield('FEEDTITLE', $res->{'FeedTitle'}, 25, 30),
				WLINK     => $cgi->textfield('FEEDLINK', $res->{'FeedLink'}, 45, 250),
				WLIMIT    => $cgi->textfield('FEEDMAX', $res->{'FeedMax'}, 3, 2),
				WCOMPACT  => $view,
				WDATE     => $date,
				WWINDOW   => $neww,
				WALTER    => $cgi->submit($config->{'TxtSubmitAlter'}),
				WRESET    => $cgi->reset($config->{'TxtResetButton'}),
				WCODE     => $cgi->textarea(
						-name    => 'FEEDCODE',
						-default => $code,
						-rows    => 3,
						-columns => 70,
						-wrap    => 'SOFT')
			 );
		}

		# Finish query
		$statement->finish();
	}

	# Default behaviour
	else
	{
		# Fetch locales
		$database->GetListFeeds(\@feeds, $param->{'ROOM'});

		# Append safety
		unshift(@feeds, $config->{'TxtNone'});


		# Generate widget
		# Locales
		$feeds = $cgi->scrolling_list('FEED',  \@feeds, $feeds[0], 10);

		# Buttons
		$buttons = $cgi->submit($config->{'TxtSubmitView'}) . '<BR><BR>' 
		  . $cgi->submit($config->{'TxtSubmitDelete'}) . '&nbsp;'
		  . $cgi->checkbox(-name=>'CONFIRM', -label=>'');

	
		# HTML
		# Beginning of form
		print $cgi->start_form();
		print $param->EmbedNormal($param->Flat(), FEED=>'True'), "\n";


		# Display template
		$tmpl->Show('TmplFeedOverview',
			MLINKS   => $links,
			WFEEDS   => $feeds,
			WACTIONS => $buttons,
			WLINK    => $cgi->textfield('FEEDSOURCE', 'http://', 45, 250),
			WCREATE  => $cgi->submit($config->{'TxtSubmitAddition'})
		 );

		# End of form
		print $cgi->end_form(), "\n";
	}
}


#####################
# Font Finder
#
# Makes the ammusing task of determining exactly what to do with what is has
# been given.  If to add or remove the value from the list et cetera.

sub FontFinder
{
	#####################
	# Data members
	my $cgi    = shift;					# CGI handle
	my $param  = shift;					# Parameter handle
	my $array  = shift;					# Array to embed
	my $realm  = shift;					# Realms information 
	my $name   = shift;					# Compare value

	my $value;						# Discovered value
	my @realm;						# Realms choices


	#####################
	# Data members

	# Initial display
	unless (defined($param->{'FONTCTRL'}))
	{
		# Only perform if one can compare
		if (defined($realm))
		{
			# Seperate into useable objects
			@realm = split(/:/, $realm);
	
			# Search
			foreach $value (@realm)
			{		
				# If matched
				if ($value eq $name)
				{
					# Remove space
					$name =~ s/\s//gs;

					# For checked
					return $cgi->checkbox(-name=>$name, -label=>'', -checked=>'ON');
				}
			}
		}

		# Nothing defined assume defaults
		else
		{
			# Remove spaces
			$name =~ s/\s//gs;

			# For checked
			return $cgi->checkbox(-name=>$name, -label=>'', -checked=>'ON');
		}
	}

	# Parameter level handling
	else
	{
		# Assign value
		$value = $name;

		# Remove spaces
		$name =~ s/\s//gs;

		
		# Array handling
		if (defined($param->{$name}))
		{
			# Push onto array
			push(@{$array}, $value);

			# For checked
			return $cgi->checkbox(-name=>$name, -label=>'', -checked=>'ON');
		}
	} 

	# Return false
	return $cgi->checkbox(-name=>$name, -label=>'');
}

#####################
# Font Handler
#
# Mostly based on the same code as the administrative script, this one will 
# allow hosts to actually make a selection of which they wish to allow for
# use.

sub FontHandler
{
	#####################
	# Data members
	my $database = shift;					# Database handle
	my $cgi      = shift;					# CGI handle
	my $param    = shift;					# Parameter reference
	my $tmpl     = shift;					# Font handler
	my $config   = shift;					# Config hash
	my $links    = shift;					# Link management

	my $audit;						# Auditing handle
	my $res;						# Results			
	my $statement;						# Database statement

	my $name;						# Font name
	my $type;						# Font type

	my $color = '';						# Font color
	my $face  = '';						# Font face
	my $size  = '';						# Font size
	
	my $rcolor;						# Realms font color
	my $rface;						# Realms font face
	my $rsize;						# Realms font size
	
	my @rcolor;						# Realms font color
	my @rface;						# Realms font face
	my @rsize;						# Realms font size


	#####################
	# Program area

	# Auditing handle
	$audit = new Ethereal::Audit($database, $cgi, $param);


	# Retreive realms choices
	($rcolor, $rface, $rsize) = $database->DataGetRealmFont($param->{'ROOM'});	


	# Query management
	# Prepare and execute
	$database->Pull(\$statement, "SELECT 
		 FontName AS \"FontName\",
		 FontType AS \"FontType\",
		 FontCode AS \"FontCode\"
		FROM  Font
		ORDER BY FontType, FontCode");


	# Cycle through fonts
	while ($res = $statement->fetchrow_hashref())
	{
		# Name handling
		$name = $res->{'FontName'};
		$type = $res->{'FontType'};

		# Display row
		# Font colour
		if ($type eq 'color')
		{
			# Display colour
			$color .= $tmpl->Pass('TmplFontColour',
				MNAME   => $res->{'FontName'},
				MCOLOR  => $res->{'FontCode'},
				WSELECT =>  FontFinder($cgi, $param, \@rcolor, $rcolor, $name)
			 );
		}

		# Font face
		elsif ($type eq 'face')
		{
			# Display template
			$face .= $tmpl->Pass('TmplFontFace',
				MNAME   => $res->{'FontName'},
				MFACE   => $res->{'FontCode'},
				WSELECT => FontFinder($cgi, $param, \@rface, $rface, $name)
			 );
		}

		# Font size
		elsif ($type eq 'size')
		{
			# Display template
			$size .= $tmpl->Pass('TmplFontSize',
				MNAME   => $res->{'FontName'},
				MSIZE   => $res->{'FontCode'},
				WSELECT => FontFinder($cgi, $param, \@rsize, $rsize, $name)
			 );
		}
	}

	# End query
	$statement->finish();


	# Write new information
	if (defined($param->{'FONTCTRL'}))
	{
		# Evaluation
		eval
		{
			# Create entries
			$rcolor = join(':', @rcolor);
			$rface  = join(':', @rface);
			$rsize  = join(':', @rsize);


			# Update database
			$database->Write("UPDATE Realm
				SET RealmFontColor=?,
				    RealmFontFace=?,
				    RealmFontSize=?
				WHERE RealmName=?",

			 $rcolor,
			 $rface,
			 $rsize,
			 $param->{'ROOM'}
			);

			# Audit entry
			$audit->FontAdjust($param->{'ROOM'}, $param->{'ROOM'});


			# Commit
			$database->Commit();
		};

		# If block failed
		if ($@)
		{
			# Issue warning
			warn("Transaction aborted: $@");

			# Undo incomplete changes
			$database->Rollback();
		}
	}


	# Form handle
	print $cgi->start_form();
	print $param->EmbedNormal(
		USER     => $param->{'USER'},
		CRYPT    => $param->{'CRYPT'},
		ROOM     => $param->{'ROOM'},
		FONT     => 'True',
		FONTCTRL => 'True'
	 );


	# Display template
	$tmpl->Show('TmplFont',
		MLINKS    => $links,
		LSTCOLOUR => $color,
		LSTFACE   => $face,
		LSTSIZE   => $size,
		WALTER    => $cgi->submit($config->{'TxtSubmitAlter'})
	 );

	# Form end
	print $cgi->end_form(), "\n";
}


#####################
# Forcasts
# 
# Allows for the host to select a weather station from which the room can have
# weather based upon.   This is the four character ICAO indicator used for 
# METAR reports.

sub Forcasts
{
	####################
	# Data members

	my $database = shift;					# Database handle
	my $cgi      = shift;					# Common gateway interface
	my $param    = shift;					# Parameter list
	my $tmpl     = shift;					# Template handler
	my $config   = shift;					# Config hash
	my $links    = shift;					# Links

	my $code;						# Code
	my $icoa;						# Station
	my $metar;						# Forcasts


	#####################
	# Program area

	# Retrieval
	($icoa, $metar) = $database->DataGet("SELECT
		  RealmICAO,
		  RealmMETAR
		FROM  Realm
		WHERE RealmName=?",

	 $param->{'ROOM'}
	);

	# Generate code
	if (defined($icoa))
	{
		# Copy and change
		$code =  $config->{'SetForecastCode'};
		$code =~ s/MID/$icoa/gs;
	}
	

	# Make sure they are defined
	$icoa  = (defined($icoa))  ? $icoa  : 'N/A';
	$metar = (defined($metar)) ? $metar : 'N/A';


	# Display
	# Star form
	print $cgi->start_form();
	print $param->EmbedNormal($param->Flat()), "\n";


	# Display template
	$tmpl->Show('TmplForcast',
		MLINKS   => $links,
		MMETAR   => $metar,
		WALTER   => $cgi->submit($config->{'TxtSubmitAlter'}),
		WSTATION => $cgi->textfield('STATION', $icoa, 4, 4),
		WEXPORT  => $cgi->textarea(
				-name    => 'FORECASTCODE',
				-default => $code,
				-rows    => 3,
				-columns => 70,
				-wrap    => 'SOFT') 
	 );

	# End of form
	print $cgi->end_form();	
}

#####################
# Gummy Handler
#
# Handles the addition and removal of gummies for realm level events.  Making
# it possible for hosts to provide more material to it's patrons outside of
# the norm.

sub GummyHandler
{
	#####################
	# Data members
	my $database = shift;					# Database handle
	my $cgi      = shift;					# CGI handle
	my $param    = shift;					# Parameter hash
	my $tmpl     = shift;					# Template handle
	my $config   = shift;					# Config hash
	my $links    = shift;					# Links management

	my $gummies;						# Widget for gummy list
	my $types;						# Widget for types

	my @gummies;						# List of gummies
	my @types    = qw(Canon Mod Super);			# Types

	#####################
	# Program area


	# Listing and display
	# Retieve list
	$database->GetList(\@gummies, "SELECT UPPER(GummyClass) || ': ' || GummyName
		FROM Gummy
		WHERE RealmName=?
		ORDER BY GummyClass, GummyName",

	 $param->{'ROOM'}
	);
	
	# Append safety
	unshift(@gummies, $config->{'TxtNone'});


	# Generate widget
	$gummies = $cgi->scrolling_list('GUMMYREM',  \@gummies, $gummies[0], 10);
	$types   = $cgi->radio_group('GUMMYTYPE', \@types, $types[0], 'true');


	# HTML
	# Beginning of form
	print $cgi->start_form();
	print $param->EmbedNormal($param->Flat(), GUMMIES=>'True'), "\n";

	# Display template
	$tmpl->Show('TmplGummy',
		MLINKS  => $links,
		WGUMMY  => $gummies,
		WDELETE => $cgi->submit($config->{'TxtSubmitDelete'}),
		WNAME   => $cgi->textfield('GUMMYNAME', '', 15, 20),
		WTYPE   => $types,
		WPATH   => $cgi->textfield('GUMMYPATH', '', 30, 200),
		WCREATE => $cgi->submit($config->{'TxtSubmitAddition'})
	 );

	# End of form
	print $cgi->end_form(), "\n";
}

#####################
# Locale Handler
#
# Handles the addition and removal of the locale settings for realms.  This
# will only apply if the realm actually selected strict mode.

sub LocaleHandler
{
	#####################
	# Data members
	my $database = shift;					# Database handle
	my $cgi      = shift;					# CGI handle
	my $param    = shift;					# Parameter hash
	my $tmpl     = shift;					# Template handle
	my $config   = shift;					# Config hash
	my $links    = shift;					# Link management

	my $locales;						# Locales widget
	my @locales;						# List of locales


	#####################
	# Program area
	
	# Fetch locales
	$database->GetListRealmLocale(\@locales, $param->{'ROOM'});

	# Append safety
	unshift(@locales, $config->{'TxtNone'});


	# Generate widget
	$locales = $cgi->scrolling_list('LOCLREM',  \@locales, $locales[0], 10);


	# HTML
	# Beginning of form
	print $cgi->start_form();
	print $param->EmbedNormal($param->Flat(), LOCALE=>'True'), "\n";

	# Display template
	$tmpl->Show('TmplLocale',
		MLINKS  => $links,
		WLOCALE => $locales,
		WDELETE => $cgi->submit($config->{'TxtSubmitDelete'}),
		WNAME   => $cgi->textfield('LOCLADD', '', 25, 50),
		WCREATE => $cgi->submit($config->{'TxtSubmitAddition'})
	 );

	# End of form
	print $cgi->end_form(), "\n";
}


#####################
# Realm Select
#
# Basic display and prompt user for a selection on which realm to adminster and
# make modifications of some type to. 

sub RealmSelect
{
	#####################
	# Data members
	my $database = shift;					# Database handle
	my $cgi      = shift;					# CGI handle
	my $param    = shift;					# Parameter hash
	my $tmpl     = shift;					# Template handle
	my $config   = shift;					# Config hash

	my $realms;						# Popup menu
	my $submit;						# Submit button

	my @realms;						# Array of realms


	#####################
	# Program area

	# Prepare of drop down
	$database->GetListSupervisorRealms(\@realms, $param->{'USER'});

	# Generate widget
	$realms = $cgi->popup_menu('ROOM', \@realms);
	$submit = $cgi->submit($config->{'TxtSubmitView'});


	# Information display
	# Start form
	print $cgi->start_form();
	print $param->EmbedNormal($param->Flat()), "\n";

	# Template listing
	$tmpl->Show('TmplRealmSelect',
		WREALMS => $realms,
		WSELECT => $submit
	 );

	# End of form and embed
	print $cgi->end_form(), "\n";	
}


#####################
# Room Ban
#
# Controls the bannings of certain individuals, this is realm specific.   As a
# result a complete ban can result in a password change that will cause the 
# user to not be able to log on any furthur

sub RoomBan
{
	#####################
	# Data members

	my $database = shift;					# Database handle
	my $cgi      = shift;					# CGI handle
	my $param    = shift;					# Parameter hash
	my $tmpl     = shift;					# Template handle
	my $config   = shift;					# Config hash
	my $links    = shift;					# Link management
	my $access   = shift;					# Access control lists

	my $statement;						# Query statement

	my $bird;						# Singular bird
	my $parole;						# Time till parole
	my $birds;						# Jailbird widget

	my @birds;						# List of jailbird puppets


	#####################
	# program area

	# Retreive pool list 
	# Prepare
	$database->Pull(\$statement, "SELECT
		 PuppeteerLogin AS \"PuppeteerLogin\",
		 getDate(RestrictionTimestamp) AS \"RestrictionTimestamp\"
		FROM  PuppeteerRestriction
		WHERE RealmName=?", 

	 $param->{'ROOM'}
	);


	# Pull a list from the pool
	while (($bird, $parole) = $statement->fetchrow())
	{ 
		# Populate array
		push(@birds, $bird . '(' . $parole . ')');	
	}

	# Finish query
	$statement->finish();


	# Addition
	unshift(@birds, $config->{'TxtNone'});

	# Format lists
	$birds = $cgi->scrolling_list('BANNED',  \@birds, $birds[0], 10);


	# Information display
	# Beginning of form
	print $cgi->start_form();
	print $param->EmbedNormal($param->Flat(), BAN=>'True'), "\n";


	# Relaxed access
	if ($access eq 'relaxed')
	{
		# Template listing
		# Applies only for those getting banned
		$tmpl->Show('TmplPoolBan',
			MLINKS    => $links,
			WCREATE   => $cgi->submit($config->{'TxtSubmitAddition'}), 
			WLOGIN    => $cgi->textfield('SUSPECT', '', 20, 30),
			WDELETE   => $cgi->submit($config->{'TxtSubmitDelete'}),
			WDURATION => $cgi->textfield('PAROLE', 0, 3, 3),
			WUSERS    => $birds,
			WREASON   => $cgi->textarea(
					-name=>'DREASON',
					-rows=>3,
					-columns=>50,
					-wrap=>'OFF',
					-default=>'')
		 );
	}

	else
	{
		# Template listing
		# Applies only for those getting banned
		$tmpl->Show('TmplPoolAccess',
			MLINKS    => $links,
			WCREATE   => $cgi->submit($config->{'TxtSubmitAddition'}), 
			WLOGIN    => $cgi->textfield('SUSPECT', '', 20, 30),
			WDELETE   => $cgi->submit($config->{'TxtSubmitDelete'}),
			WDURATION => $cgi->textfield('PAROLE', 0, 3, 3),
			WREASON   => $cgi->hidden('DREASON',''),
			WUSERS    => $birds
		 );
	}

	# End of form
	print $cgi->end_form(), "\n";
}


#####################
# Room Canon
#
# Allows for the host to add in canon handles, meaning handles that
# are restricted to the realm specifically and probably already exist
# on the system.

sub RoomCanon
{
	#####################
	# Data members
	my $database = shift;					# Database handle
	my $cgi      = shift;					# CGI handle
	my $param    = shift;					# Parameter hash
	my $tmpl     = shift;					# Template handle
	my $config   = shift;					# Config hash
	my $links    = shift;					# Link management

	my $audit;						# Auditing
	my $statement;						# Database statement

	my $name;						# Massaged name
	my $canon;						# Hash of cannon handles

	my $current;						# Current canons
	my $list;						# Listing of entries

	my %realm;						# Hash for realm tags


	#####################
	# Program area

	# Populate hash
	$database->GetHashRealm(\%realm);

	# Define instance
	$audit = new Ethereal::Audit($database, $cgi, $param);


	# Display canon handles
	$database->Pull(\$statement, "SELECT
		 PuppetName     AS \"PuppetName\",
		 PuppeteerLogin AS \"PuppeteerLogin\"
		FROM  Puppet
		WHERE PuppetPool=?
		ORDER BY PuppetName",

	 $param->{'ROOM'}
	);

	# Checks
	if ($canon = $statement->fetchrow_hashref())
	{
		# Initialize
		$list = '';
		
		# Cycle through
		do
		{
			# Determin names
			$name =  $canon->{'PuppetName'};
			$name =~ s/\W//gs;


			# Hide
			if (defined($param->{$name}))
			{
				# Evaluate block
				eval
				{
				 	# Remove canon handle
					$database->Write("DELETE FROM Puppet
						WHERE PuppetName=?
						  AND PuppetPool=?",
	
					 $canon->{'PuppetName'},
					 $param->{'ROOM'}
					);	
	
					# Cleanup
					$param->Cleanup($name);
	
					# Record in audit
					$audit->CanonDel($canon->{'PuppetName'}, $param->{'ROOM'});
	
	
					# Commit changes
					$database->Commit();
				};
	
				# If block failed
				if ($@)
				{
					# Issue warning
					warn("Transaction aborted: $@");
	
					# Undo incomplete changes
					$database->Rollback();
				}
			} 	

			# Append for display
			else
			{
				# Complete the template
				$list =~ $tmpl->Pass('TmplCanonList',
					MHANDLE => $canon->{'PuppetName'},
					MLOGIN  => $canon->{'PuppeteerLogin'},
					WSELECT => $cgi->checkbox(-name=>$name, -label=>'')
				 );
			}
		
	
		# Loop until complete list
		} while ($canon = $statement->fetchrow_hashref());


		# Populate current listing
		$current = $tmpl->Pass('TmplCanonCurrent',
			MLIST   => $list,
			WDELETE => $cgi->submit($config->{'TxtSubmitDelete'})
		 );
	}

	# Shooting blanks
	else
	{
		# Blank out
		$current = '';
	}

	# End query
	$statement->finish();


	# Puppet Addition
	# Start form
	print $cgi->start_form();
	print $param->EmbedNormal($param->Flat());


	# Display template
	$tmpl->Show('TmplCanon',
		MLINKS   => $links,
		WCREATE  => $cgi->submit($config->{'TxtSubmitAddition'}),
		MCURRENT => $current,
		WNAME    => $cgi->textfield('CCHAR', '', 15, 30),
		WUSER    => $cgi->textfield('CUSER', '', 15, 10)
	 );

	# End of form
	print $cgi->end_form();
}



#####################
# Room Design
#
# Simply put, this section will be handling the design aspects of a realm
# this deals with the splash page, the header and footer used o the pages
# as well as modifications to the communications panel.

sub RoomDesign
{
	#####################
	# Data members
	my $database = shift;					# Database handle
	my $cgi      = shift;					# CGI handle
	my $param    = shift;					# Parameter hash
	my $tmpl     = shift;					# Template handle
	my $config   = shift;					# Config hash
	my $url      = shift;					# Self referencing link
	my $links    = shift;					# Links management

	my $statement;						# Query statement
	my $res;						# Results handle
	
	my $path     = $database->{'DBROOM'};			# Realm tags
	my $search   = $database->{'SEARCH'};			# Calling site
	my $room;

	my $offline;						# Offline links
	my $online;						# Online links
	my $short;						# Short name

	my $irange;						# Inline range selection
	my $irealm;						# Inline realms selection

	my $fields;						# Useable format
	my $text;						# Text area text

	my %docs;						# Documents list
	my %copy;						# Copy tie

	#####################
	# Program area

	# Cleanup
	$param->Cleanup('DRANGE','DTEXT');


	# Generate inlines
	$irealm = $param->EmbedInline($param->Flat(), DRANGE=>'Realm');
	$irange = $param->EmbedInline($param->Flat(), DRANGE=>'Offline');
	
	# Create proper links
	# Fetch design names
	$database->Pull(\$statement, "SELECT
		  t.TagName       AS \"TagName\",
		  t.TagType       AS \"TagType\",
		  t.TagData       AS \"TagData\",
		  d.DesignContent AS \"RealmData\"
		 FROM Tag t
		 LEFT OUTER JOIN RealmDesign d
		   ON (t.TagName=d.TagName AND d.RealmName=?)
		WHERE t.TagName LIKE 'Realm%'
	       	   OR t.TagName LIKE 'Offline%'
		ORDER BY t.TagName",

       	 $param->{'ROOM'}
	);
	
	# Quick check
	if ($res = $statement->fetchrow_hashref())
	{
		# Initialize
		$offline = '';
		$online  = '';
		
	
		# Loop and generate
		do
		{
			# Capture short name
			$short = $res->{'TagName'};
			$short =~ s/^(Offline|Realm)//;
			
			# Sort accordingly
			# Normal features
			if ($res->{'TagName'} =~ /^Realm/)
			{
				# Online realms
				$online .= $tmpl->Pass('TagTemplates',
					MLINK => $url . $irealm . '&DFILD=' . $res->{'TagName'},
					MNAME => $short
				 );
			}

			# Offline mode
			else
			{
				# Online realms
				$offline .= $tmpl->Pass('TagTemplates',
					MLINK => $url . $irange . '&DFILD=' . $res->{'TagName'},
					MNAME => $short
				 );
			}
	
			# Pupulate hash
			$docs{$res->{'TagName'}} = (defined($res->{'RealmData'}))
       			 ? $res->{'RealmData'}
 			 : $res->{'TagData'};

		} while ($res = $statement->fetchrow_hashref());
	}

	# Finish with updates
	$statement->finish();


	# Update of BerkelyDB databases
	# Check for need
	if ((defined($path))
	  && (-d $path))
  	{
		# Prepare tie
		$room =  lc($param->{'ROOM'});
		$room =~ s/\W//gs;
		
		# Pull defaults
		my $def = new DB_File::BTREEINFO;

		# Adjust file
		$path .= '/' . $search . '.' . $room;


		# Tie variable
		tie(%copy, 'DB_File', $path, O_RDWR|O_CREAT, 0666, $def)
      		 or die('Error linking with $param->{ROOM} hash: ', $!);


		# Copy over completely
		%copy = %docs;

		# Untie and go
		untie %copy;


		# Set permissions to something sane
		chmod(0666, $path);
	}
	

	# Change of behaviour
	# Field structure
	if (defined($param->{'DFILD'}))
	{
		# Pull template information
		$text = $database->FetchRealm($param->{'DFILD'}, $param->{'ROOM'});


		if (defined($text))
		{
			# Fetch short name
			$short =  $param->{'DFILD'};
			$short =~ s/^(Realm|Offline)(.*)/$1\/$2/;
			
			# Start form
			print $cgi->start_form();
			print $param->EmbedNormal($param->Flat(),
				DESIGN => 'True',
				DFILD  => $param->{'DFILD'}
			 ), "\n";	
		
			# Display normal design
			$tmpl->Show('TmplDesignText',
				LSTONLINE  => $online,
				LSTOFFLINE => $offline,
				MLINKS     => $links,
				MTEMPLATE  => $short,
				WSUBMIT    => $cgi->submit($config->{'TxtSubmitButton'}),
				WRESET     => $cgi->submit($config->{'TxtResetButton'}),
				WDESCR     => $cgi->textarea(
						-name    => 'DTEXT',
						-rows    => 20,
						-columns => 70,
						-wrap    => 'OFF',
						-default => $text)
			 );


			# End of form 
			print $cgi->end_form(), "\n";
		}
	}

	# The basics
	else
	{
		# Display normal design
		$tmpl->Show('TmplDesignBlank',
			MLINKS     => $links,
			LSTONLINE  => $online,
			LSTOFFLINE => $offline,
		 );
	}
}


#####################
# Room Moderators
#
# Allows for the addition of moderators into the system itself.  While not 
# overly necessary as no power is actually gotten from this.  It will
# allow to see more trusted people within the realms.

sub RoomMods
{
	#####################
	# Data members
	my $database = shift;					# Database handle
	my $cgi      = shift;					# CGI handle
	my $param    = shift;					# Parameter hash
	my $tmpl     = shift;					# Template handler
	my $config   = shift;					# Config hash
	my $links    = shift;					# Link management

	my $option;						# Option handler

	my $acl;						# Access control widget
	my $moderators;						# List of moderator puppets
	my $res;						# Discovered results
	my $statement;						# Queried information

	my $login;						# Returned login
	my $access;						# Access number
	my $trans;						# Translated access
	my $super    = 16;					# Supervisory access level

	my @acl;						# Access control list
	my @moderators;						# List of moderator puppets


	#####################
	# program area

	# Create instance
	$option = new Ethereal::Option();
	$option->Split(\@acl, $config->{'OptACL'});
	

	# Retreive list of moderators
	$database->Pull(\$statement, "SELECT
		  PuppeteerLogin AS \"PuppeteerLogin\",
		  RealmACL       AS \"RealmACL\"
                FROM  RealmSupervisor
                WHERE RealmName=?
		  AND RealmACL < ?
		ORDER BY PuppeteerLogin",

	 $param->{'ROOM'},
	 $super
	);

	# Loop and determine
	while (($login, $access) = $statement->fetchrow())
	{
		# Cleanup
		$trans = '';

		# Convoluted system
		if (($access - 8) >= 0) { $access -= 8; $trans = "S$trans"; }
		if (($access - 4) >= 0) { $access -= 4; $trans = "J$trans"; }
		if (($access - 2) >= 0) { $access -= 2; $trans = "D$trans"; }
		if (($access - 1) >= 0) { $access -= 1; $trans = "A$trans"; }

		# Create login name
		$login = "$login ($trans)" if (length($trans) > 0);

		# Append
		push(@moderators, $login);	
	}

	# Finish
	$statement->finish();

	# Append safety measure
	unshift(@moderators, $config->{'TxtNone'});


	# Format widgets
	$moderators = $cgi->scrolling_list('MODREM',  \@moderators, $moderators[0], 10);
	$acl        = $cgi->checkbox_group(-name=>'MODACL', -values=>\@acl, -linebreak=>'true', -default=>$acl[0]);


	# HTML
	# Beginning of form
	print $cgi->start_form();
	print $param->EmbedNormal($param->Flat(), MOD=>'True'), "\n";

	# Display template
	$tmpl->Show('TmplModerator',
		MLINKS  => $links,
		WMODS   => $moderators,
		WDELETE => $cgi->submit($config->{'TxtSubmitDelete'}),
		WCREATE => $cgi->submit($config->{'TxtSubmitAddition'}),
		WLOGIN  => $cgi->textfield('MODADD', '', 20, 30),
		WROLES  => $acl
	 );

	# End of form
	print $cgi->end_form(), "\n";
}


#####################
# Room Picture Ban
#
# This manages the basic display for the addition and removal of pictures from
# the banned list.  Banned implies that the image cannot be used again on the
# system.

sub RoomPicture
{
	#####################
	# Data members
	my $database = shift;					# Database handle
	my $cgi      = shift;					# CGI handle
	my $param    = shift;					# Parameter handle
	my $tmpl     = shift;					# Template handle
	my $config   = shift;					# Config hash
	my $links    = shift;					# Links

	my $pics;						# List of pics
	my @pics;						# List of pics


	#####################
	# program area

	# Retreive list of images
	$database->GetListImageBan(\@pics);
	unshift(@pics, $config->{'TxtNone'});

	# Create widget
	$pics = $cgi->scrolling_list('PICREM', \@pics, $pics[0], 10);


	# HTML
	# Beginning of form
	print $cgi->start_form();
	print $param->EmbedNormal($param->Flat(), GRAPHS=>'True'), "\n";

	# Display template
	$tmpl->Show('TmplPicture',
		MLINKS    => $links,
		WPICTURES => $pics,
		WDELETE   => $cgi->submit($config->{'TxtSubmitDelete'}),
		WCREATE   => $cgi->submit($config->{'TxtSubmitAddition'}),
		WLINK     => $cgi->textfield('PICADD', '', 50, 255)
	 );

	# End of form
	print $cgi->end_form(), "\n";
}


#####################
# RoomView
# 
# Will allow the supervisors to view the selected room and make changes as necessary
# and will probably make the changes themselves.

sub RoomView
{
	#####################
	# Data members
	my $database = shift;					# Database handle
	my $cgi      = shift;					# CGI handle
	my $param    = shift;					# Parameter hash
	my $tmpl     = shift;					# Template handle
	my $config   = shift;					# Config hash
	my $admin    = shift;					# Administrative access
	my $links    = shift;					# Linky
 
	my $comm;						# Comm panel handle
	my $option;						# Option handler
	my $statement;						# Database statement

	my $access;						# Access control
	my $banner;						# Banner widget
	my $color;						# Colour selection default
	my $dateloc;						# Location of timestamp
	my $harass;						# Harassment mode
	my $lang;						# Language restriction
	my $locale;						# Locale display
	my $genre;						# Realm genre
	my $rating;						# Realm rating
	my $realm;						# Realm information
	my $tags;						# Tag enforcement
	my $verified;						# Verification enforcement

	my $lstadm  = '';					# Administrative list
	my $lsthp   = '';					# Homepage sample list

	my @optcolr;						# Colour choices 
	my @optgnre;						# Genre choices
	my @optrate;						# Rating choices

	my @optaccs = qw(Relaxed Restricted List);		# Access control
	my @optbner = qw(Display Exempt);			# Banner display
	my @optdate = qw(Bottom Top Removed);			# Timestamp location
	my @opthars = qw(Yes No);				# Harassment mode
	my @optlang = qw(Relaxed Restricted);			# Language access
	my @optlocl = qw(None Strict Variable);			# Locale display
	my @opttags = qw(Relaxed Restricted);			# Tag enforcement
	my @optvery = qw(Yes No);				# Verification enforcement


	#####################
	# Program area

	# Create instances
	$comm   = new Ethereal::Comm($database, $cgi);
	$option = new Ethereal::Option();

	# Pull colours
	@optcolr = $comm->FontSelectColor($param);

	# Pull genre and rating
	$database->GetListRealmGenre(\@optgnre);
	$option->Split(\@optrate, $config->{'SetRating'});


	# Populate realm information hashref
	$database->Pull(\$statement, "SELECT
		 RealmName         AS \"RealmName\",
		 RealmHomepage     AS \"RealmHomepage\",
		 RealmDescription  AS \"RealmDescription\",
		 RealmGenre        AS \"RealmGenre\",
		 RealmRating       AS \"RealmRating\",
		 RealmLanguage     AS \"RealmLanguage\", 
		 RealmFilter       AS \"RealmFilter\",
		 RealmAccess       AS \"RealmAccess\", 
		 RealmVerified     AS \"RealmVerified\", 
		 RealmHarass       AS \"RealmHarass\",
		 RealmTags         AS \"RealmTags\", 
		 RealmLocale       AS \"RealmLocale\", 
		 RealmBanner       AS \"RealmBanner\", 
		 RealmTextColour   AS \"RealmTextColour\",
		 RealmImageHeight  AS \"RealmImageHeight\", 
		 RealmImageWidth   AS \"RealmImageWidth\",
		 RealmImageSize    AS \"RealmImageSize\",
		 RealmDateLoc      AS \"RealmDateLoc\",
		 RealmDateFormat   AS \"RealmDateFormat\",
		 RealmAltBoard     AS \"RealmAltBoard\",
		 RealmArchive      AS \"RealmArchive\",
		 RealmBoard        AS \"RealmBoard\"
		FROM Realm 
		WHERE RealmName=?",

	 $param->{'ROOM'}
	);

	# Retreive and finish
	$realm = $statement->fetchrow_hashref();
	$statement->finish();

	# Prepare radiobuttons
	$access   = $cgi->radio_group('ACCESS', \@optaccs, ucfirst($realm->{'RealmAccess'}), 'true');
	$banner   = $cgi->radio_group('BANNER', \@optbner, ucfirst($realm->{'RealmBanner'}), 'true');
	$dateloc  = $cgi->radio_group('DATLOC', \@optdate, ucfirst($realm->{'RealmDateLoc'}), 'true');
	$harass   = $cgi->radio_group('HARASS', \@opthars, ucfirst($realm->{'RealmHarass'}), 'true');
	$lang     = $cgi->radio_group('LANG',   \@optlang, ucfirst($realm->{'RealmLanguage'}), 'true');
	$locale   = $cgi->radio_group('LOCL',   \@optlocl, ucfirst($realm->{'RealmLocale'}), 'true');
	$tags     = $cgi->radio_group('TAGS',   \@opttags, ucfirst($realm->{'RealmTags'}), 'true');
	$verified = $cgi->radio_group('VERIFY', \@optvery, ucfirst($realm->{'RealmVerified'}), 'true');

	# Different widget
	$color    = $cgi->popup_menu('COLOR',  \@optcolr, $realm->{'RealmTextColour'});
	$genre    = $cgi->popup_menu('GENRE',  \@optgnre, $realm->{'RealmGenre'});
	$rating   = $cgi->popup_menu('RATING', \@optrate, $realm->{'RealmRating'});

        # Assign values
        $realm->{'RealmAltBoard'}    = (defined($realm->{'RealmAltBoard'}))    ? $realm->{'RealmAltBoard'} : 'http://';
        $realm->{'RealmHomepage'}    = (defined($realm->{'RealmHomepage'}))    ? $realm->{'RealmHomepage'} : 'http://';
        $realm->{'RealmBoard'}       = (defined($realm->{'RealmBoard'}))       ? $realm->{'RealmBoard'} : '';
	$realm->{'RealmArchive'}     = (defined($realm->{'RealmArchive'}))     ? $realm->{'RealmArchive'} : '';
	$realm->{'RealmDescription'} = (defined($realm->{'RealmDescription'})) ? $realm->{'RealmDescription'} : '';
	$realm->{'RealmFilter'}      = (defined($realm->{'RealmFilter'}))      ? $realm->{'RealmFilter'} : '****';


	# Pre-generation
	# Administrative
	if ($admin eq 'yes')
	{
		# Generate template
		# Passed onto another template
		$lstadm = $tmpl->Pass('TmplOverviewAdmin',
			WACCESS => $access,
			WBANNER => $banner,
			WVERIFY	=> $verified,
			WHARASS => $harass
		 );
	}

	# Sample code
	if ($realm->{'RealmHomepage'} !~ /^http:\/\/$/)
	{
		# Retrieve
		my ($sample) = $database->DocumentGetWebring();
		my ($ident)  = $cgi->escape($realm->{'RealmName'});

		# Manipulate and escape
		$sample =  $cgi->escapeHTML($sample);
		$sample =~ s/MREALM/$ident/gs;
		$sample =~ s/\n/<BR>/gs;

		# Passed onto another template
		$lsthp => $tmpl->Pass('TmplOverviewWebring',
			MSAMPLE => $sample
		);
	}


	# HTML display
	# Start form
	print $cgi->start_form();
	print $param->EmbedNormal($param->Flat(), MAIN=>'True'), "\n";	


	# Display template
	$tmpl->Show('TmplOverview',
		MLINKS    => $links,
		MREALM    => $realm->{'RealmName'},
		LSTADMIN  => $lstadm,
		LSTRING   => $lsthp,
		WLOCALE   => $locale,
		WHFORMAT  => $tags,
		WSTAMP    => $dateloc,
		WSFORMAT  => $cgi->textfield('DFORMAT', $realm->{'RealmDateFormat'}, 30, 30),
		WLANGUAGE => $lang,
		WLFORMAT  => $cgi->textfield('FILTER', $realm->{'RealmFilter'}, 10, 10),
		WFONT     => $color,
		WGENRE    => $genre,
		WRATE     => $rating,
		WHEIGHT   => $cgi->textfield('IMGHEIGHT', $realm->{'RealmImageHeight'}, 3, 3),
		WWIDTH    => $cgi->textfield('IMGWIDTH',  $realm->{'RealmImageWidth'}, 3, 3),
		WSIZE     => $cgi->textfield('IMGSIZE',  $realm->{'RealmImageSize'}, 3, 3),
		WHOMEPAGE => $cgi->textfield('HOMEPAGE', $realm->{'RealmHomepage'}, 70, 255),
		WARCHIVE  => $cgi->textfield('ARCHIVE', $realm->{'RealmArchive'}, 70, 45),
		WADMIN    => $cgi->textfield('BOARDS', $realm->{'RealmBoard'}, 70, 45),
		WBOARD    => $cgi->textfield('ALTBOARD', $realm->{'RealmAltBoard'}, 70, 255),
		WSUBMIT   => $cgi->submit($config->{'TxtSubmitButton'}),
		WRESET    => $cgi->submit($config->{'TxtResetButton'}),
		WDESCR    => $cgi->textarea(
				-name    => 'RDESCR',
				-rows    => 3,
				-columns => 51,
				-wrap    => 'OFF',
				-default => $realm->{'RealmDescription'})
	 );
		
	# End of form 
	print $cgi->end_form(), "\n";
}


#####################
# Room Warn
#
# Will list who is in a realm, and allow a moderator or supervisor to send a
# warning that will be recorded.  This allows hosts to see if someone is causing
# a bit of mischeif.

sub RoomWarn
{
	#####################
	# Data members
	my $database = shift;					# Database handle
	my $cgi      = shift;					# CGI handle
	my $param    = shift;					# Parameter handle
	my $tmpl     = shift;					# Template handle
	my $config   = shift;					# Config hash
	my $links    = shift;					# Links

	my $who;						# Formatted widget
	my @whos;						# Who is online


	#####################
	# program area

	# Retreive list of banned and suspects
	# Manual due to unique purpose
	$database->GetList(\@whos, "SELECT PuppeteerLogin
		FROM Post
		WHERE RealmName=?
		  AND PostTimestamp > getInterval('10 Minutes')
		  AND PuppetName IS NOT NULL",

	 $param->{'ROOM'});

	# Append for safety
	unshift(@whos, $config->{'TxtNone'});

	# Create widget
	$who = $cgi->scrolling_list('WLOGIN',  \@whos, $whos[0], 10);


	# HTML
	# Beginning of form
	print $cgi->start_form();
	print $param->EmbedNormal($param->Flat(), WARN=>'True'), "\n";

	# Display template
	$tmpl->Show('TmplWarn',
		MLINKS   => $links,
		WHANDLES => $who,
		WWARN    => $cgi->submit($config->{'TxtWarn'}),
		WREASON  => $cgi->textarea(
				-name    => 'WMSG',
				-default => $config->{'TxtWarnMsg'},
				-rows    => 3,
				-columns => 50,
				-wrap    => 'SOFT')
	 );

	# End of form
	print $cgi->end_form();
}

#####################
# Room Word
#
# Controls word restrictions, as a result one can add or remove words for which
# such words will be replaced from posts by ****.  Simplistic but effective.

sub RoomWord
{
	#####################
	# Data members
	my $database = shift;					# Database handle
	my $cgi      = shift;					# CGI handle
	my $param    = shift;					# Parameter handle
	my $tmpl     = shift;					# Template handle
	my $config   = shift;					# Config hash
	my $links    = shift;					# Links

	my $words;						# List of words
	my @words;						# List of words	


	#####################
	# program area

	# Retreive list of banned and suspects
	$database->GetListWords(\@words, $param->{'ROOM'});
	unshift(@words, $config->{'TxtNone'});

	# Create widget
	$words = $cgi->scrolling_list('WORDREM',  \@words, $words[0], 10);


	# HTML
	# Beginning of form
	print $cgi->start_form();
	print $param->EmbedNormal($param->Flat(), WORDS=>'True'), "\n";

	# Display template
	$tmpl->Show('TmplWords',
		MLINKS  => $links,
		WPOTTY  => $words,
		WDELETE => $cgi->submit($config->{'TxtSubmitDelete'}),
		WCREATE => $cgi->submit($config->{'TxtSubmitAddition'}),
		WWORD   => $cgi->textfield('WORDADD', '', 10, 10)
	 );

	# End of form
	print $cgi->end_form(), "\n";
}


#####################
# Room Write
# 
# Will handle the writing of information gathered from the forms. 

sub RoomWrite
{
	#####################
	# Data members
	my $database = shift;					# Database handle
	my $cgi      = shift;					# CGI handle
	my $param    = shift;					# Parameter handle
	my $config   = shift;					# Config hash
	my $access   = shift;					# Access realms
	my $admin    = shift;					# Administrative access
	my $acl      = shift;					# Access control list

	my $audit;						# Audit handler
	my $filter;						# Filter handle
	my $option;						# Option handle
	my $rss;						# RSS parser
	my $send;						# Mail sending

	my $acls;						# Specific total for ACL
	my $choice;						# Choice of the matter
	my $count;						# Counter
	my $time;						# Time values
	my $super    = 16;					# Supervisory access

	my $gummy;						# Gummy name
	my $puppet;						# Puppet name
	my $puppeteer;						# Puppeteers name

	my %system;						# System hash


	#####################
	# Program area
	
	# Audit handle
	$audit  = new Ethereal::Audit($database, $cgi, $param);
	$filter = new Ethereal::Filter();

	# Evaluation
	eval
	{
		# Word restrictions
		if (defined($param->{'WORDS'}))
		{
			# Removal
			if ((defined($param->{$config->{'TxtSubmitDelete'}}))
			 && ($param->{'WORDREM'} ne $config->{'TxtNone'}))
			{ 
				# Remove entry
				$database->Write("DELETE FROM Restriction 
					WHERE RealmName=?
					  AND RestrictionType='word'
					  AND RestrictionValue=?", 

				 $param->{'ROOM'},
				 $param->{'WORDREM'}
				);

				# Audit entry
				$audit->WordRem($param->{'WORDREM'}, $param->{'ROOM'});
			}

			# Addition
			elsif (defined($param->{$config->{'TxtSubmitAddition'}}))
			{ 
				# Only if a word
				if (($param->{'WORDADD'} !~ /^\W+/sg)
				 && (length($param->{'WORDADD'}) > 2))
				{
					# Create new entry
					$database->Write("INSERT INTO Restriction 
						(RealmName,
						 RestrictionType,
						 RestrictionValue)
						VALUES (?,'word',?)",

				 	 $param->{'ROOM'},
				 	 $param->{'WORDADD'}
					);

					# Audit entry
					$audit->WordAdd($param->{'WORDADD'}, $param->{'ROOM'});
				}
			}	


			# Cleanup
			$param->Cleanup('WORDS');
		}


		# Board Topics
		elsif (defined($param->{'TOPICS'}))
		{
			# Removal of posts and topic
			if (defined($param->{'TDELETE'}))
			{
				# Remove the topic
				$database->Write("DELETE FROM BoardTopic 
					WHERE RealmName=?
					  AND BoardTopic=?", 

				 $param->{'ROOM'},
				 $param->{'TOPIC'}
				);

				# Audit entry
				$audit->TopicAdd($param->{'TOPIC'}, $param->{'ROOM'});
			}

			# Addition of topics
			elsif ((defined($param->{$config->{'TxtSubmitAddition'}}))
			 && ($param->{'TADD'} !~ /^system/i))
			{
				# Verify validity
				if (($param->{'TADD'} !~ /^\W+/sg)
				 && (length($param->{'TADD'}) > 2))
				{
					# Clear out HTML and truncate
					$param->{'TDESCR'} = $filter->StripHTML($param->{'TDESCR'}); 
					$param->{'TDESCR'} = substr(DBI::neat($param->{'TDESCR'}, 2002), 1, -1);

					# Final definition
					$param->{'TDESCR'} = (length($param->{'TDESCR'}) > 15)
						? $param->{'TEDIT'}
						: undef;

					# Create
					$database->Write("INSERT INTO BoardTopic 
						(BoardTopic,
						 RealmName,
						 BoardDescription)
						VALUES (?,?,?)", 

				  	 $param->{'TADD'},
				 	 $param->{'ROOM'},
					 $param->{'TDESCR'}
					);

					# New home
					$database->Write("UPDATE Board
						SET BoardTopic=?
						WHERE BoardTopic='system'
						  AND RealmName=?",

					 $param->{'TADD'},
					 $param->{'ROOM'}
					);


					# Audit entry
					$audit->TopicAdd($param->{'TADD'}, $param->{'ROOM'});

					
					# Set for proper display
					$param->{'TOPIC'} = $param->{'TADD'};
				}
			}	

			# Editing of words
			elsif ((defined($param->{$config->{'TxtSubmitAlter'}}))
			 && ($param->{'TOPIC'}) ne $config->{'TxtTopic'})
			{
				print "-ALTER-";
				
				# Clear out HTML and truncate
				$param->{'TEDIT'} = $filter->StripHTML($param->{'TEDIT'}); 
				$param->{'TEDIT'} = substr(DBI::neat($param->{'TEDIT'}, 2002), 1, -1);

				# Final definition
				$param->{'TEDIT'} = (length($param->{'TEDIT'}) > 15)
					? $param->{'TEDIT'}
					: undef;


				# Update database
				$database->Write("UPDATE BoardTopic
					SET BoardDescription=?
					WHERE BoardTopic=?
					  AND RealmName=?",

				 $param->{'TEDIT'},
				 $param->{'TOPIC'},
				 $param->{'ROOM'}
				);

				# Audit entry
				$audit->TopicAdd($param->{'TADD'}, $param->{'ROOM'});
			}

			# Cleanup
			$param->Cleanup('CONFIRM','TOPICS','TDESCR','TDELETE');
		}

	
		# Picture restrictions
		elsif (defined($param->{'GRAPHS'}))
		{
			# Removal
			if ((defined($param->{"$config->{'TxtSubmitDelete'}"}))
			 && ($param->{'GRAPH'} ne $config->{'TxtNone'}))
			{ 
				# Remove
				$database->Write("DELETE FROM Restriction 
					WHERE RealmName='system'
					  AND RestrictionType='picture'
					  AND RestrictionValue=?", 

				 $param->{'PICREM'}
				);

				# Audit entry
				$audit->PicRem($param->{'PICREM'}, $param->{'ROOM'});
			}

			# Addition
			elsif (defined($param->{"$config->{'TxtSubmitAddition'}"}))
			{ 
				# Only if a word
				if (length($param->{'PICADD'}) > 10)
				{
					# Ban the image
					$database->Write("INSERT INTO Restriction 
						(RealmName,
						 RestrictionType,
						 RestrictionValue)
						VALUES ('system','picture',?)", 

				 	 $param->{'PICADD'}
					);

					# Clear all instances of image
					$database->Write("UPDATE Puppet
						SET PuppetPic=NULL,
						    PuppetPicHeight=NULL,
						    PuppetPicWidth=NULL
						WHERE PuppetPic ILIKE '$param->{PICADD}%\'");


					# Audit entry
					$audit->PicAdd($param->{'PICADD'}, $param->{'ROOM'});
				}
			}	

			# Cleanup
			$param->Cleanup('PICS');
		}

		# RSS Feeds
		elsif (defined($param->{'FEED'}))
		{
			# Addition
			if ((defined($param->{'FEEDSOURCE'}))
			 && ($param->{'FEEDSOURCE'}) =~ /^http:\/\/.+/i)
			{
				# Pull file
				my $content = get($param->{'FEEDSOURCE'});

				# Feed exists
				if (defined($content))
				{
					# Create instance
					$rss = new XML::RSS;
					
					# Parse file
					$rss->parse($content);	


					# Pull information
					my $title = $rss->{'channel'}->{'title'};
					my $link  = $rss->{'channel'}->{'link'};


					# Final check
					if ((defined($title))
					 && (defined($link)))
				 	{
						# Create a new entry
						$database->Write("INSERT INTO Feed
							(RealmName,
							 FeedSource,
							 FeedTitle,
							 FeedLink,
							 FeedArchive)
							VALUES (?,?,?,?,?)",

						 $param->{'ROOM'},
						 $param->{'FEEDSOURCE'},
						 $title,
						 $link,
						 $content
						);

						# Record activity
						$audit->FeedAdd($title);
					}
				}

				# Cleanup
				$param->Cleanup('FEEDSOURCE');
			}
		
			# Removal
			elsif ((defined($param->{'FEED'}))
			 && (defined($param->{$config->{'TxtSubmitDelete'}}))
			 && (defined($param->{'CONFIRM'}))
		 	 && ($param->{'FEED'} ne $config->{'TxtNone'}) 
			 && ($param->{'FEED'} ne 'True'))
			{
				# Pull ID and Topic
				$param->{'FEED'} =~ /(\d+) - (.+)/;

				# Assign
				my $num   = $1;
				my $title = $2;


				# Remove entry
				$database->Write("DELETE FROM Feed
					WHERE RealmName=?
					  AND FeedID=?",

				 $param->{'ROOM'},
				 $num
				);

				# Record activity
				$audit->FeedDel($param->{'FEED'});


				# Cleanup
				$param->Cleanup('CONFIRM');

				# Reset
				$param->{'FEED'} = 'True';
			}

			# Modification of information
			elsif ((defined($param->{'FID'}))
			 && (length($param->{'FEEDTITLE'}) > 3)
			 && ($param->{'FEEDLINK'} =~ /^http:\/\/.+/i))
			{
				# Ensure definition
				$param->{'FEEDMAX'} = ($param->{'FEEDMAX'} =~ /^\d{1,2}$/) ? $param->{'FEEDMAX'} : 0;
				
				# Commit changes
				$database->Write("UPDATE Feed
					SET FeedTitle=?,
					    FeedLink=?,
					    FeedMax=?,
					    FeedCompactView=?,
					    FeedShowDate=?,
					    FeedNewWindow=?
					WHERE FeedID=?
					  AND RealmName=?",

				 $param->{'FEEDTITLE'},
				 $param->{'FEEDLINK'},
				 $param->{'FEEDMAX'},
				 lc($param->{'FEEDVIEW'}),
				 lc($param->{'FEEDDATE'}),
				 lc($param->{'FEEDNEWW'}),
				 $param->{'FID'},
				 $param->{'ROOM'}
			 	);

				# Record activity
				$audit->FeedChange($param->{'FID'} . ' - ' . $param->{'FEEDTITLE'});

				# Cleanup
				$param->Cleanup('FID');
			}
		}
		
		# For banning handling
		elsif (defined($param->{'BAN'}))
		{
			# Removal of person
			if ((defined($param->{"$config->{'TxtSubmitDelete'}"}))
			 && ($param->{'BANNED'} ne $config->{'TxtNone'}))
			{
				# Strip date information
				$param->{'BANNED'} =~ s/\s\(\d{2}-\w{3}-\d{4}\)//s;
				
				# Proper formatting
				($puppeteer) = $database->DataGetPuppeteerExistence($param->{'BANNED'});
			
				# Remove if he is a valid user
			 	if (defined($puppeteer))
				{	
					# Remove
					$database->Write("DELETE FROM PuppeteerRestriction 
						WHERE PuppeteerLogin=?
						  AND RealmName=?", 

					 $puppeteer,
					 $param->{'ROOM'}
					);
	
					# Audit entry
					$audit->BanRem($puppeteer, $param->{'ROOM'});
				}
			}

			# Addition
			elsif (defined($param->{"$config->{'TxtSubmitAddition'}"}))
			{
				# Determine existence
				($puppeteer) = $database->DataGetPuppeteerExistence($param->{'SUSPECT'});

				# Verification
				if ((length($param->{'SUSPECT'}) > 2)
				 && ($param->{'SUSPECT'} !~ /^\s/)
				 && (defined($puppeteer)))
				{
					# Make sure the timestamp is valid
					$param->{'PAROLE'} = ($param->{'PAROLE'} =~ /\d+/) ? $param->{'PAROLE'} : 0;
					$param->{'PAROLE'} = ($param->{'PAROLE'} < 1000)   ? $param->{'PAROLE'} : 1000;


					# Determine when one is paroled
					$time = ($param->{'PAROLE'} < 1) ? "('01/01/3000 00:00:00')::TIMESTAMP" : "(LOCALTIMESTAMP + INTERVAL '$param->{PAROLE} days')::TIMESTAMP";

					# Write and record addition
					$database->Write("INSERT INTO PuppeteerRestriction 
						(PuppeteerLogin,
						 RealmName,
						 RestrictionTimestamp)	
						VALUES (?,?,$time)",

					 $puppeteer, 
					 $param->{'ROOM'}
					); 

					# Email notificiation
					if ($access ne 'list')
					{
						# Link with hash
						$database->GetHashSystem(\%system);

						# Pull values
						my ($banned) = $database->DocumentGetMailBanned();
						my ($email)  = $database->DataGetPuppeteerEmail($puppeteer);
						my ($name)   = $database->DataGetPuppeteerName($puppeteer);
						my ($parole) = $database->DataGet("SELECT getDate($time) AS \"Time\"");

						# Default reason
						$param->{'DREASON'} = (length($param->{'DREASON'}) > 3) ? $param->{'DREASON'} : $system{'SetBannedDefault'};


						# Mail sending
						# Initiate handle
						$send = tie(*MAIL, 'Ethereal::Mail', $database);

						# Initial settings
						$send->SetSubject("You have been banned from $param->{ROOM}");
						$send->SetSearch(
							BANDATE    => $parole,
							BANREASON  => $param->{'DREASON'},
							FULLNAME   => $name,
							ROOMNAME   => $param->{'ROOM'},
							SERVERNAME => $system{'SetInfoServer'},
							USERNAME   => $puppeteer
						 );
						$send->ToggleWrap();

						# Recipients
						$send->AddTo("$name <$email>");
						$send->AddBCC("$system{SetBannedName} <$system{SetBannedAddress}>");


						# Message
						print MAIL $banned;

						# Close and send
						close(MAIL);
					}


					# Record in audit
					$audit->BanAdd($puppeteer, $param->{'ROOM'});
				}
			}

			# Cleanup
			$param->Cleanup('BAN','TIMESPAN');
		}


		# Handling of canon handles
		elsif (defined($param->{'CANON'}))
		{
			# Addition
			if ((defined($param->{'CCHAR'})) 
			 && (defined($param->{'CUSER'})) 
			 && (length($param->{'CCHAR'}) > 2))
			{
				# Determine existence
				($puppeteer) = $database->DataGetPuppeteerExistence($param->{'CUSER'});
				($puppet)    = $database->DataGetPuppetExistence($param->{'CCHAR'}, $param->{'CUSER'});


				# Everything in place
				if ((defined($puppeteer))
				 && (!defined($puppet)))
				{
					# Find gummy default
					($gummy) = $database->DataGet("SELECT GummyPath
						FROM Gummy
						WHERE GummyClass=?
						  AND RealmName=?
						ORDER BY GummyName LIMIT 1", 
						
					 'canon',
					 'system'
				 	);


					# Create handle
					$database->Write("INSERT INTO Puppet
						(PuppetName,
						 PuppeteerLogin,
						 PuppetPool,
						 PuppetGummy)
						VALUES(?,?,?,?)",

					 $param->{'CCHAR'},
					 $puppeteer,
					 $param->{'ROOM'},
					 $gummy
					);


					# Audit entry
					$audit->CanonAdd($param->{'CCHAR'}, $param->{'ROOM'});
				}
			}


			# Cleanup
			$param->Cleanup('CDEL','CCHAR','CUSER');
		}


		# Gummy handler
		elsif (defined($param->{'GUMMY'}))
		{
			# Locale additions
			if ((defined($param->{$config->{'TxtSubmitAddition'}}))
			 && (defined($param->{'GUMMYNAME'}))
			 && (defined($param->{'GUMMYPATH'}))
			 && (defined($param->{'GUMMYTYPE'}))
			 && (length($param->{'GUMMYNAME'}) > 2)
			 && (length($param->{'GUMMYPATH'}) > 5))
			{
				# Insert locale
				$database->Write("INSERT INTO Gummy
					(GummyName,
					 GummyClass,
					 GummyPath,
					 RealmName)
					VALUES (?,?,?,?)",

				 $param->{'GUMMYNAME'},
				 lc($param->{'GUMMYTYPE'}),
				 $param->{'GUMMYPATH'},
				 $param->{'ROOM'}
				);

				# Auditing entry
				$audit->LocalGummyAdd($param->{'LOCLADD'}, $param->{'ROOM'});

				# Cleanup
				$param->Cleanup('GUMMYNAME', 'GUMMYTYPE', 'GUMMYPATH');
			}

			# Locale removals
			elsif  ((defined($param->{$config->{'TxtSubmitDelete'}}))
		 	 && ($param->{'GUMMYREM'} ne $config->{'TxtNone'}))
			{
				# Verify for proper string format
				if ($param->{'GUMMYREM'} =~ /([A-Z]+): (.+)/)
				{
					# Pull class andname
					my $gclass = $1;
					my $gname  = $2;


					# Remove locale
					$database->Write("DELETE FROM Gummy
						WHERE GummyName=?
						  AND GummyClass=?
						  AND RealmName=?",

					 $gname,
					 lc($gclass),
					 $param->{'ROOM'}
					);

					# Auditing entry
					$audit->LocalGummyRem($param->{'GUMMYREM'}, $param->{'ROOM'});

					# Cleanup
					$param->Cleanup('GUMMYREM');
				}
			}
		}


		# Local handler
		elsif (defined($param->{'LOCALE'}))
		{
			# Locale additions
			if ((defined($param->{$config->{'TxtSubmitAddition'}}))
			 && (defined($param->{'LOCLADD'}))
			 && (length($param->{'LOCLADD'}) > 2))
			{
				# Insert locale
				$database->Write("INSERT INTO RealmLocale
					(RealmLocale,
					 Realmname)
					VALUES (?,?)",

				 $param->{'LOCLADD'},
				 $param->{'ROOM'}
				);

				# Auditing entry
				$audit->LocaleAdd($param->{'LOCLADD'}, $param->{'ROOM'});

				# Cleanup
				$param->Cleanup('LOCLADD');
			}

			# Locale removals
			elsif  ((defined($param->{$config->{'TxtSubmitDelete'}}))
		 	 && ($param->{'LOCLADD'} ne $config->{'TxtNone'}))
			{
				# Remove locale
				$database->Write("DELETE FROM RealmLocale
					WHERE RealmLocale=?
					  AND RealmName=?",

				 $param->{'LOCLREM'},
				 $param->{'ROOM'}
				);

				# Auditing entry
				$audit->LocaleRem($param->{'LOCLREM'}, $param->{'ROOM'});

				# Cleanup
				$param->Cleanup('LOCLREM');
			}
		}


		# Handling of design
		elsif ((defined($param->{'DESIGN'}))
		    && (defined($param->{$config->{'TxtSubmitButton'}})))
		{
			# Determine Realm Information
			if (length($param->{'DTEXT'}) > 15)
			{
				# Verify for existence
				($choice) = $database->DataGet("SELECT RealmName
					FROM RealmDesign
					WHERE RealmName=?
					  AND TagName=?", 

				 $param->{'ROOM'},
				 $param->{'DFILD'}
				);
			

				# Update
				if (defined($choice))
				{
					# Update information
					$database->Write("UPDATE RealmDesign
						SET DesignContent=?
						WHERE RealmName=?
						  AND TagName=?",

					 $param->{'DTEXT'},
					 $param->{'ROOM'}, 
 					 $param->{'DFILD'}
					);
				}

				# Create instance
				else
				{
					# Insert information
					$database->Write("INSERT INTO RealmDesign
						(DesignContent,
						 TagName,
						 RealmName)
						VALUES (?,?,?)",

					 $param->{'DTEXT'},
					 $param->{'DFILD'},
					 $param->{'ROOM'}
					);
				}
			}

			# Delete entry
			else
			{
				# Remove row and use defaults
				$database->Write("DELETE FROM RealmDesign
					WHERE RealmName=?
					  AND TagName=?",

				 $param->{'ROOM'},
				 $param->{'DFILD'}
				);
			}

			# Audit entry
			$audit->InfoChange($param->{'DFILD'}, $param->{'ROOM'});


			# Cleanup
			$param->Cleanup('DTEXT');
		}

		# For moderator handling
		elsif (defined($param->{'MOD'}))
		{
			# Removal of person
			if ((defined($param->{$config->{'TxtSubmitDelete'}}))
			 && ($param->{'MODREM'} ne $config->{'TxtNone'}))
			{
				# Strip additional material
				$param->{'MODREM'} =~ s/^(.*)\s\(\w+\)$/$1/;

	
				# Initial handle
				# Remove from realm
				$database->Write("DELETE FROM RealmSupervisor
					WHERE PuppeteerLogin=?
					  AND RealmName=?",

				 $param->{'MODREM'},
				 $param->{'ROOM'}
				);

				# Audit entry
				$audit->ModsRem($param->{'MODREM'}, $param->{'ROOM'});


				# Retreive positions
				($count) = $database->DataGetModCount($param->{'MODREM'}, $super);

				# Moderator bit handling
				if ($count < 1)
				{
					# Update database
					$database->Write("UPDATE Puppeteer 
						SET   PuppeteerModPrivs='no' 
						WHERE PuppeteerLogin=?", 

					 $param->{'MODREM'}
					);

					# Reset gummy
					$database->Write("UPDATE Puppet 
						SET   PuppetGummy=NULL 
						WHERE PuppeteerLogin=?",

					 $param->{'MODREM'}
					);
				}
			}

			# Addition
			elsif (defined($param->{$config->{'TxtSubmitAddition'}}))
			{

				# Verification
				if ((length($param->{'MODADD'}) > 2)
				 && ($param->{'MODADD'} !~ /^\s/))
				{
					# Retrieve positions
					($count) = $database->DataGetModCount($param->{'MODADD'}, $super);


					# Determine ACL number
					# Create instance
					$option = new Ethereal::Option();

					# Initial assignment
					$acls = 0;

					# Determine
					foreach $choice (@{$acl})
					{
						# Determine number and add
						$acls += $option->FindACL($config->{'OptACL'}, $choice);
					}


					# Change moderator bit
					if ($count < 1)
					{
						# Change host bit
						$database->Write("Update Puppeteer 
							SET   PuppeteerModPrivs='yes' 
							WHERE PuppeteerLogin=?",

						 $param->{'MODADD'}
						);
					}
				
					# Add room specific entry
					$database->Write("INSERT INTO RealmSupervisor
						(PuppeteerLogin,
						 RealmName,
						 RealmACL)
						VALUES(?,?,?)",
 
					 $param->{'MODADD'},
					 $param->{'ROOM'},
					 $acls);

					# Record in audit
					$audit->ModsAdd($param->{'MODADD'}, $param->{'ROOM'});
				}
			}

			# Cleanup
			$param->Cleanup('MOD','MODACL');
		}

		# Update main information
		elsif (defined($param->{'MAIN'}))
		{
			# Determine validity
			$param->{'ARCHIVE'}  = ($param->{'ARCHIVE'} =~ /^\w(\w|\.|\-)*?\@(\w|\-)*?\..*?/s) ? $param->{'ARCHIVE'} : undef;
			$param->{'BOARDS'}   = ($param->{'BOARDS'}  =~ /^\w(\w|\.|\-)*?\@(\w|\-)*?\..*?/s) ? $param->{'BOARDS'} : undef;
			$param->{'ALTBOARD'} = ((length($param->{'ALTBOARD'}) > 12) && ($param->{'ALTBOARD'} =~ /^http:\/\//is)) ? $param->{'ALTBOARD'} : undef;
			$param->{'HOMEPAGE'} = ((length($param->{'HOMEPAGE'}) > 12) && ($param->{'HOMEPAGE'} =~ /^http:\/\//is)) ? $param->{'HOMEPAGE'} : undef;
			$param->{'FILTER'}   = (length($param->{'FILTER'}) > 1) ? $param->{'FILTER'} : '****';


			# Realm description
			$param->{'RDESCR'} = $filter->StripHTML($param->{'RDESCR'});
			$param->{'RDESCR'} = substr(DBI::neat($param->{'RDESCR'}, 252), 1, -1);
			$param->{'RDESCR'} = (length($param->{'RDESCR'}) > 50) ? $param->{'RDESCR'} : undef;

			# Massage description
			if (defined($param->{'RDESCR'}))
			{
				# Truncate
				$param->{'RDESCR'} = substr($param->{'RDESCR'}, 0, 250);
		
				# Replace quotes
				$param->{'RDESCR'} =~ s/'/`/gs;
				$param->{'RDESCR'} =~ s/\n/ /gs;
				$param->{'RDESCR'} =~ s/\r/ /gs;
			}


			# Banner exemption
			if ($admin eq 'yes')
			{
				# Special update
				$database->Write("UPDATE Realm
				    	SET RealmAccess=?, 
					    RealmBanner=?,
					    RealmHarass=?,
					    RealmVerified=?
					WHERE RealmName=?",

			 	 lc($param->{'ACCESS'}), 
				 lc($param->{'BANNER'}),
				 lc($param->{'HARASS'}), 
				 lc($param->{'VERIFY'}),
				 $param->{'ROOM'});

				# Cleanup
				$param->Cleanup('BANNER');
			}


			# Ensure numeric
			$param->{'IMGHEIGHT'} = ($param->{'IMGHEIGHT'} =~ /^\d+$/) ? $param->{'IMGHEIGHT'} : 0;
			$param->{'IMGWIDTH'}  = ($param->{'IMGWIDTH'}  =~ /^\d+$/) ? $param->{'IMGWIDTH'}  : 0;
			$param->{'IMGSIZE'}   = ($param->{'IMGSIZE'}   =~ /^\d+$/) ? $param->{'IMGSIZE'}   : 0;

			# Font formatting
			$param->{'DFORMAT'}   = (length($param->{'DFORMAT'}) > 5)  ? $param->{'DFORMAT'}  : 'DD-Mon-YYYY, HH24:MI:SS TZ';


			# Update primary information
			$database->Write("UPDATE Realm 
				SET RealmHomepage=?,
				    RealmDescription=?,
				    RealmGenre=?,
				    RealmRating=?,
				    RealmLanguage=?, 
				    RealmFilter=?, 
				    RealmTags=?,
				    RealmLocale=?,
				    RealmDateLoc=?,
				    RealmDateFormat=?,
				    RealmTextColour=?,
				    RealmImageHeight=?, 
				    RealmImageWidth=?,
				    RealmImageSize=?,
				    RealmArchive=?,
				    RealmBoard=?,
				    RealmAltBoard=?
				WHERE RealmName=?",

			 $param->{'HOMEPAGE'},
			 $param->{'RDESCR'},
			 $param->{'GENRE'},
			 $param->{'RATING'},
			 lc($param->{'LANG'}), 
			 $param->{'FILTER'},
			 lc($param->{'TAGS'}),
			 lc($param->{'LOCL'}),
			 lc($param->{'DATLOC'}),
			 $param->{'DFORMAT'},
			 $param->{'COLOR'},
			 $param->{'IMGHEIGHT'}, 
			 $param->{'IMGWIDTH'},
			 $param->{'IMGSIZE'},
			 $param->{'ARCHIVE'},
			 $param->{'BOARDS'},
			 $param->{'ALTBOARD'},
			 $param->{'ROOM'});

			# Audit entry
			$audit->RealmDetails($param->{'ROOM'}, $param->{'ROOM'});
			
			# Cleanup
			$param->Cleanup('MAIN');
		}

		# Weather forcasts
		elsif (defined($param->{'FORCAST'}))
		{
			if (defined($param->{$config->{'TxtSubmitAlter'}}))
			{
				# Data Members
				my $metar;
				my $station = $param->{'STATION'};


				# Assign values
				if ($station =~ /^\w{4}/)
				{
					# Convert to uppercase
					$station = uc($station);

					# Find Forcast
					my $forc = new Ethereal::Weather();
					($station, $metar) = $forc->Get($station);

				}
				else
				{
					# Assign to undefined
					$metar   = undef;
					$station = undef;
				}


				# Modification
				# Change database entry
				$database->Write("UPDATE Realm
					SET RealmICAO=?,
					    RealmMETAR=?
					WHERE RealmName=?",

				 $station,
				 $metar,
				 $param->{'ROOM'}
				);

				# Record audit change
				$audit->Forcast($station, $param->{'ROOM'});	
			}
		}

		# Warnings
		elsif (defined($param->{'WARN'}))
		{
		 	# There is a point to do this?		
			if ((defined($param->{'WLOGIN'}))
			 && ($param->{'CHAR'} ne $config->{'TxtNone'}))
			{
				# Pull notice
				my ($warn) = $database->FetchRealmWarning($param->{'ROOM'});

				# Search and replace
				$warn =~ s/MNOTICE/$param->{'WMSG'}/gs;
				$warn =~ s/MLOGIN/$param->{'WLOGIN'}/gs;


				# Create post
				$database->Write("INSERT INTO Post
					(RealmName,
					 PostTo,
					 PostFullFormat,
					 PostImagelessFormat,
					 PostPartialFormat)
					VALUES (?,?,?,?,?)",
				
				 $param->{'ROOM'},
				 $param->{'WLOGIN'},
				 $warn,
				 $warn,
				 $warn);

				# Record audit change
				$audit->Warn($param->{'WLOGIN'}, $param->{'ROOM'});	
			}
		}

		# Commit
		$database->Commit();
	};

	# If block failed
	if ($@)
	{
		# Issue warning
		warn("Transaction aborted: $@");

		# Undo incomplete changes
		$database->Rollback();
	}
}


#####################
# Spacer
#
# Simple takes in what it requires to determin if a space has to be appened at the end
# of the link bar to precent ungodly vertical scroll.

sub Spacer
{
	#####################
	# Data members
	my $size   = shift;					# Current size
	my $limit  = shift;					# What to push at
	my $word   = shift;					# Pulled word

	my $next;						# Next determined size
	my $pass;						# Return value
	
	#####################
	# Program area
	
	# What will it be next
	$next = ${$size} + length($word);

	# Return
	# Determine return
	$pass = (int(${$size} / $limit) < int($next / $limit))
	  ? '<br>'
	  : '';

	# Set for next pass
	${$size} = $next;
	  
	return $pass;
}
