#!/usr/bin/perl
use lib '/home/martin/ethereal/mgmt';

#################################################################################
# Created       : Martin Foster
# Modified      : 03-May-2005
#################################################################################
#
# Feed - RSS feed parser making use of Ethereal Realms for archiving and storage
#        of preferences on how to proceede.
# Copyright (C) 2000-2006  Martin Foster
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# Author of this script can be contacted at the following:
#       E-Mail  : martin@ethereal-realms.org
#       Address : 29-26503 TWP RD 511
#                 Spruce Grove, Alberta
#                 T7Y 1G4
#
#################################################################################

use LWP::Simple qw(!head);					# Simple HTTP handling
use CGI qw(:standard);						# Common gateway interface
use CGI::Carp qw(fatalsToBrowser);				# CGI Error logs
use XML::RSS;							# RSS feed parser
use strict;							# Strict variable enforcement

use Ethereal::Database;						# Database handler
use Ethereal::Param;						# Parameter control


#################################################################################
# Data Members
#################################################################################
my $cgi;							# Common gateway interface handle
my $database;							# Database handle
my $param;							# Parameter list
my $rss;							# RSS feed handle

my $res;							# Results set
my $statement;							# Database statement


my $concat;							# Concatenated line
my $date;							# Pulled date
my $feed;							# Feed identifier
my $max;							# Maximum fields
my $target;							# New window code

my $count = 0;							# Counter
my $parse = 0;							# Parser counts


#################################################################################
# Program Area
#################################################################################

	# Initial handles
	$cgi      = new CGI;
	$database = new Ethereal::Database();

	# Connect and fetch
	$database->Connect($cgi);	

	# Parameter handling
	$param = new Ethereal::Param($database, $cgi);
	$param->GetParam();
	

	# Display header
	print $cgi->header('text/javascript');

	# Validity
	if ((defined($param->{'FID'}))
	 && ($param->{'FID'} =~ /^\d+$/))
 	{
		# Evaluate block
		eval
		{
			# Pull information
			# Prepare and execute
			$database->Pull(\$statement, "SELECT
				 FeedTitle        AS \"FeedTitle\",
				 FeedLink         AS \"FeedLink\",
				 FeedSource       AS \"FeedSource\",
				 FeedMax          AS \"FeedMax\",
				 FeedCompactView  AS \"FeedCompactView\",
				 FeedShowDate     AS \"FeedShowDate\",
				 FeedNewWindow    AS \"FeedNewWindow\",
				 FeedArchive      AS \"FeedArchive\",
				 getCurrent(FeedTimestamp)  AS \"FeedCurrent\"
				FROM Feed
				WHERE FeedID=?
				FOR UPDATE OF Feed",

			 $param->{'FID'}
			);

			# Litmus test
			# Results are valid
			if ($res = $statement->fetchrow_hashref())
			{
				# Create instance
				$rss = new XML::RSS;


				# Current archive
				# Stale archive
				if ($res->{'FeedCurrent'} != 1)
				{
					# Pull list
					my $content = get($res->{'FeedSource'});

					# Parse RSS contnet
					$rss->parse($content);

					# Set parse count
					$parse++;


					# Validity check
					# Assign
					my $title = $rss->{'channel'}->{'title'};
					my $link  = $rss->{'channel'}->{'link'};

					# Update if valid
					if ((defined($title))
					 && (defined($link)))
				 	{
						# Update archive
						$database->Write("UPDATE Feed
							SET FeedArchive=?,
							    FeedTimestamp=LOCALTIMESTAMP
							WHERE FeedID=?",

						 $content,
						 $param->{'FID'}
						);
					}

					# Sound error
					else
					{
						# Display error message
						print "document.write('<b>[ERROR: Cannot parse feed.  Using archive]</b>');";

						# Reset parse
						$parse--;
					}
				}

				# Parse document
				$rss->parse($res->{'FeedArchive'}) if ($parse < 1);


				# Grabbing date
				$date = $rss->{'channel'}->{'dc'}->{'date'}  if (defined($rss->{'channel'}->{'dc'}->{'date'}));
				$date = $rss->{'channel'}->{'lastBuildDate'} if (defined($rss->{'channel'}->{'lastBuildDate'}));

				# Target handler
				$target = ($res->{'FeedTarget'} eq 'yes') ? "TARGET=\"_blank\"" : "";
				
				# Define true max 
				$max    = ($res->{'FeedMax'} != 0) ? $res->{'FeedMax'} : 100;


				# Begin line
				# Alignmnet
				$concat = "<div class=\"FEED_DIV\">";

				# Title bar
				$concat = "<table class=\"FEED_TABLE\">"
				 . "<tr>"
				 . "<td class=\"FEED_TITLE\">"
				 . "<a href=\"$res->{FeedLink}\" CLASS=\"FEED_LINK\" $target>$res->{FeedTitle}</a>"
				 . "</td>"
				 . "</tr>";


				# Loop and add in
				foreach my $item (@{$rss->{'items'}})
				{
					# Increment
					$count++;
					
					# Checks
					next unless defined($item->{'title'}) && defined($item->{'link'});
					last if ($count > $max);


					# Start table
					$concat = $concat
					 . "<tr>"
					 . "<td class=\"FEED_ITEM\">"
					 . "<a href=\"$item->{link}\" CLASS=\"ITEM_LINK\" $target>$item->{title}</a>";

					# Possible description
					if (($res->{'FeedCompactView'} eq 'no')
					 && (defined($item->{'description'})))
				 	{
						# Append the description only
						$concat . $concat . "<br><p class=\"ITEM_DESCR\">$item->{description}</p>";
					}

					# Tail end
					$concat = $concat
				 	. "</td>"
				 	. "</tr>";
				}

				# Date if needed
				if ($res->{'FeedShowDate'} eq 'yes')
				{
					# Date updated bar
					$concat = $concat
					 . "<tr>"
					 . "<td class=\"FEED_DATE\">$date</td>"
					 . "</tr>";
				}

				# Close up
				$concat . $concat
				. "</table>"
				. "</div>";


				# Safety
				$concat =~ s/'/\\'/gs;

				# Escape characters and display
				print "document.write('" . $concat . "');";
			}

			# Feed is missing
			else
			{
				# Display error message
				print "document.write('<b>[ERROR: Feed does not exist]</b>');";
			}


			# Commit changes
			$database->Commit();

		};
		
		# If block failed
		if ($@)
		{
			# Issue warning
			print "document.write('<b>[ERROR: Transaction error]</b>');";

			# Undo incomplete changes
			$database->Rollback();
		}

		# Finish
		$statement->finish();
	}

	# Sound error
	else
	{
		# Through javascript
		print "document.write('<b>[ERROR: Invalid parameters]</b>');";
	}
