#!/usr/bin/perl
use lib '/home/martin/ethereal/mgmt';

################################################################################
# Created       : Martin Foster
# Modified      : 24-Nov-2005
################################################################################
#
# FAQ - Script part of Ethereal Realms used to generate a list of Frequently
#       asked Questions used for the site.
# Copyright (C) 2000-2006  Martin Foster
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# Author of this script can be contacted at the following:
#       E-Mail  : martin@ethereal-realms.org
#       Address : 29-26503 TWP RD 511
#                 Spruce Grove, Alberta
#                 T7Y 1G4
#
#################################################################################

use CGI qw(:standard);						# Common gateway interface
use CGI::Carp qw(fatalsToBrowser);				# CGI Error logs
use strict;							# Strict variable enforcement

use Ethereal::Database;						# Database handler
use Ethereal::Table;						# Table handler
use Ethereal::Template;						# Template handler


################################################################################
# Data Members
################################################################################
my $cgi;							# Common gateway interface handle
my $database;							# Database handle
my $tmpl;							# Template handle

my $scriptname;							# Simply the script name
my $scriptpara;							# Scripted parameters

my %faq;							# FAQ hash

################################################################################
# Program Area
################################################################################

	# Initial handles
	$cgi      = new CGI;
	$database = new Ethereal::Database();

	# Connect and fetch
	$database->Connect($cgi);

	# Link with hash
	$database->GetHashFaq(\%faq);

	# Initial hash
	$tmpl  = new Ethereal::Template(\%faq);

	# Pull script name
	$scriptname = $cgi->url(-relative=>1);

	# Round about way
	if ((length($scriptname) < 2)
	 || ($scriptname =~ /\.(shtml|html)$/i))
	{
		# Assume this is not a real script name
		# Get name and only end
		$scriptname = $0;
		$scriptname =~ s/.*\/(.*)$/$1/gs;
	}

	# File header
	print $cgi->header();

	# Display Frequently Asked Questions (FAQ)
	FaqHandler($database, $tmpl, \%faq, $scriptname);


################################################################################
# Sub-Routines
################################################################################

#####################
# Faq Handler
#
# Makes use of heavy templating in the same way that the portal for regular
# users and contributors are.   Making the FAQ a very malleable templating
# script.

sub FaqHandler
{
	#####################
	# Data members
	my $database = shift;					# Database handle
	my $tmpl     = shift;					# Template handle
	my $gal      = shift;					# Gallery hash
	my $option   = shift;					# Scriptname and deicisons

	my $res;						# Results
	my $statement;						# Database query
	
	my $anchor;						# Anchored group
	my $compo;						# Composit index
	my $faq;						# Generated FAQ template
	my $index;						# Index set
	my $group;						# Singular group
	my $groups;						# Formatted list
	my $qanda;						# Exchange of questions

	my $count;						# Counter
	my $ogroup;						# Old group
	
	my @groups;						# List of groups


	#####################
	# Program Area

	# Group list
	# Pull list of groups
	$database->GetList(\@groups, "SELECT FaqGroup
		FROM Faq
		GROUP BY FaqGroup
		ORDER BY FaqGroup");

	# Initialize
	$groups = '';
	
	# Run through and generate
	foreach $group (@groups)
	{
		# Generate anchor
		$anchor =  $gal->{'LnkFAQ'} . '#' . uc($group);
		$anchor =~ s/\s/_/gs;
		
		# Add in all appropriate material
		$groups = $groups . $tmpl->Pass('TmplGroupList',
			MANCHOR => $anchor,
			MNAME   => ucfirst($group)
		 );
	}


	# Questions and Answers
	# Generate query and execute
	$database->Pull(\$statement, "SELECT
		 FaqNum      AS \"FaqNum\",
		 FaqQuestion AS \"FaqQuestion\",
		 FaqGroup    AS \"FaqGroup\",
		 FaqAnswer   AS \"FaqAnswer\"
		FROM Faq
		WHERE FaqAnswer IS NOT NULL
		ORDER BY FaqGroup, FaqQuestion");
	
	# Safety
	if ($res = $statement->fetchrow_hashref())
	{
		# Initialize
		$count  = 0;
		$ogroup = $res->{'FaqGroup'};

		# Text init
		$compo  = '';
		$qanda  = '';
		$index  = '';

		# Loop and collect
		do
		{
			# Group name and title
			if (($ogroup ne $res->{'FaqGroup'})
			 or ($count < 1))
			{
				# Generate anchor
				$anchor = uc($res->{'FaqGroup'});
				$anchor =~ s/\s/_/gs;


				# Previous set
				# Throw in buffer if applicable
				if ($count > 0)
				{
					# Complete index
					$compo = $compo . $tmpl->Pass('TmplQuestionIndex',
						MANCHOR => $gal->{'LnkFAQ'} . '.#' . $anchor,
						MNAME   => ucfirst($ogroup),
						LSTINDEX => $index
					 );

					 # Reset for clear pass
					 $index = '';
				}

				# Pass on spacer if needed
				$qanda = $qanda . $tmpl->Pass('TmplQuestionSpacer') if ($count > 0);

				# Actual title
				$qanda = $qanda . $tmpl->Pass('TmplQuestionGroup',
					MANCHOR => "<A NAME=\"$anchor\"></A>",
					MNAME   => ucfirst($res->{'FaqGroup'})
				 );

				 # Set for next pass
				 $ogroup = $res->{'FaqGroup'};
			}

			# List of questions and answers
			$index = $index . $tmpl->Pass('TmplQuestionIndexItem',
				MANCHOR   => $gal->{'LnkFAQ'} . '#Q' . $res->{'FaqNum'},
				MQUESTION => $res->{'FaqQuestion'}
			 );

			# Main set of questions and answers
			$qanda = $qanda . $tmpl->Pass('TmplQuestionList',
				MANCHOR   => "<A NAME=\"Q$res->{FaqNum}\"></A>",
				MQUESTION => $res->{'FaqQuestion'},
				MANSWER   => $res->{'FaqAnswer'}
			 );

			# Increment
			$count++;

		} while ($res = $statement->fetchrow_hashref());


		# Previous set
		# Throw in buffer if applicable
		if ((defined($index))
		 && (length($index) > 5))
		{
			# Complete index
			$compo = $compo . $tmpl->Pass('TmplQuestionIndex',
				MANCHOR => $gal->{'LnkFAQ'} . '#' . $anchor,
				MNAME   => ucfirst($ogroup),
				LSTINDEX => $index
			 );
		}


		# End query
		$statement->finish();
	}
	
	# Generate master template
	$tmpl->Show('TmplMaster',
		LSTINDEX     => $compo,
		LSTGROUPS    => $groups,
		LSTQUESTIONS => $qanda
	 );
}
